# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.stemmer._paice_husk.

Paice-Husk Stemmer
"""

from typing import Dict, Optional, Tuple

from ._stemmer import _Stemmer

__all__ = ['PaiceHusk']


class PaiceHusk(_Stemmer):
    """Paice-Husk stemmer.

    Implementation of the Paice-Husk Stemmer, also known as the Lancaster
    Stemmer, developed by Chris Paice, with the assistance of Gareth Husk

    This is based on the algorithm's description in :cite:`Paice:1990`.

    .. versionadded:: 0.3.6
    """

    _rule_table = {
        6: {
            'ifiabl': ((False, 6, None, True),),
            'plicat': ((False, 4, 'y', True),),
        },
        5: {
            'guish': ((False, 5, 'ct', True),),
            'sumpt': ((False, 2, None, True),),
            'istry': ((False, 5, None, True),),
        },
        4: {
            'ytic': ((False, 3, 's', True),),
            'ceed': ((False, 2, 'ss', True),),
            'hood': ((False, 4, None, False),),
            'lief': ((False, 1, 'v', True),),
            'verj': ((False, 1, 't', True),),
            'misj': ((False, 2, 't', True),),
            'iabl': ((False, 4, 'y', True),),
            'iful': ((False, 4, 'y', True),),
            'sion': ((False, 4, 'j', False),),
            'xion': ((False, 4, 'ct', True),),
            'ship': ((False, 4, None, False),),
            'ness': ((False, 4, None, False),),
            'ment': ((False, 4, None, False),),
            'ript': ((False, 2, 'b', True),),
            'orpt': ((False, 2, 'b', True),),
            'duct': ((False, 1, None, True),),
            'cept': ((False, 2, 'iv', True),),
            'olut': ((False, 2, 'v', True),),
            'sist': ((False, 0, None, True),),
        },
        3: {
            'ied': ((False, 3, 'y', False),),
            'eed': ((False, 1, None, True),),
            'ing': ((False, 3, None, False),),
            'iag': ((False, 3, 'y', True),),
            'ish': ((False, 3, None, False),),
            'fuj': ((False, 1, 's', True),),
            'hej': ((False, 1, 'r', True),),
            'abl': ((False, 3, None, False),),
            'ibl': ((False, 3, None, True),),
            'bil': ((False, 2, 'l', False),),
            'ful': ((False, 3, None, False),),
            'ial': ((False, 3, None, False),),
            'ual': ((False, 3, None, False),),
            'ium': ((False, 3, None, True),),
            'ism': ((False, 3, None, False),),
            'ion': ((False, 3, None, False),),
            'ian': ((False, 3, None, False),),
            'een': ((False, 0, None, True),),
            'ear': ((False, 0, None, True),),
            'ier': ((False, 3, 'y', False),),
            'ies': ((False, 3, 'y', False),),
            'sis': ((False, 2, None, True),),
            'ous': ((False, 3, None, False),),
            'ent': ((False, 3, None, False),),
            'ant': ((False, 3, None, False),),
            'ist': ((False, 3, None, False),),
            'iqu': ((False, 3, None, True),),
            'ogu': ((False, 1, None, True),),
            'siv': ((False, 3, 'j', False),),
            'eiv': ((False, 0, None, True),),
            'bly': ((False, 1, None, False),),
            'ily': ((False, 3, 'y', False),),
            'ply': ((False, 0, None, True),),
            'ogy': ((False, 1, None, True),),
            'phy': ((False, 1, None, True),),
            'omy': ((False, 1, None, True),),
            'opy': ((False, 1, None, True),),
            'ity': ((False, 3, None, False),),
            'ety': ((False, 3, None, False),),
            'lty': ((False, 2, None, True),),
            'ary': ((False, 3, None, False),),
            'ory': ((False, 3, None, False),),
            'ify': ((False, 3, None, True),),
            'ncy': ((False, 2, 't', False),),
            'acy': ((False, 3, None, False),),
        },
        2: {
            'ia': ((True, 2, None, True),),
            'bb': ((False, 1, None, True),),
            'ic': ((False, 2, None, False),),
            'nc': ((False, 1, 't', False),),
            'dd': ((False, 1, None, True),),
            'ed': ((False, 2, None, False),),
            'if': ((False, 2, None, False),),
            'ag': ((False, 2, None, False),),
            'gg': ((False, 1, None, True),),
            'th': ((True, 2, None, True),),
            'ij': ((False, 1, 'd', True),),
            'uj': ((False, 1, 'd', True),),
            'oj': ((False, 1, 'd', True),),
            'nj': ((False, 1, 'd', True),),
            'cl': ((False, 1, None, True),),
            'ul': ((False, 2, None, True),),
            'al': ((False, 2, None, False),),
            'll': ((False, 1, None, True),),
            'um': ((True, 2, None, True),),
            'mm': ((False, 1, None, True),),
            'an': ((False, 2, None, False),),
            'en': ((False, 2, None, False),),
            'nn': ((False, 1, None, True),),
            'pp': ((False, 1, None, True),),
            'er': ((False, 2, None, False),),
            'ar': ((False, 2, None, True),),
            'or': ((False, 2, None, False),),
            'ur': ((False, 2, None, False),),
            'rr': ((False, 1, None, True),),
            'tr': ((False, 1, None, False),),
            'is': ((False, 2, None, False),),
            'ss': ((False, 0, None, True),),
            'us': ((True, 2, None, True),),
            'at': ((False, 2, None, False),),
            'tt': ((False, 1, None, True),),
            'iv': ((False, 2, None, False),),
            'ly': ((False, 2, None, False),),
            'iz': ((False, 2, None, False),),
            'yz': ((False, 1, 's', True),),
        },
        1: {
            'a': ((True, 1, None, True),),
            'e': ((False, 1, None, False),),
            'i': ((True, 1, None, True), (False, 1, 'y', False)),
            'j': ((False, 1, 's', True),),
            's': ((True, 1, None, False), (False, 0, None, True)),
        },
    }  # type: Dict[int, Dict[str, Tuple[Tuple[bool, int, Optional[str], bool], ...]]]  # noqa: E501

    def _has_vowel(self, word: str) -> bool:
        for char in word:
            if char in {'a', 'e', 'i', 'o', 'u', 'y'}:
                return True
        return False

    def _acceptable(self, word: str) -> bool:
        if word and word[0] in {'a', 'e', 'i', 'o', 'u'}:
            return len(word) > 1
        return len(word) > 2 and self._has_vowel(word[1:])

    def _apply_rule(
        self,
        word: str,
        rule: Tuple[bool, int, Optional[str], bool],
        intact: bool,
        terminate: bool,
    ) -> Tuple[str, bool, bool, bool]:
        old_word = word
        only_intact, del_len, add_str, set_terminate = rule
        # print(word, word[-n:], rule)

        if (not only_intact) or (intact and only_intact):
            if del_len:
                word = word[:-del_len]
            if add_str:
                word += add_str
        else:
            return word, False, intact, terminate

        if self._acceptable(word):
            return word, True, False, set_terminate
        else:
            return old_word, False, intact, terminate

    def stem(self, word: str) -> str:
        """Return Paice-Husk stem.

        Parameters
        ----------
        word : str
            The word to stem

        Returns
        -------
        str
            Word stem

        Examples
        --------
        >>> stmr = PaiceHusk()
        >>> stmr.stem('assumption')
        'assum'
        >>> stmr.stem('verifiable')
        'ver'
        >>> stmr.stem('fancies')
        'fant'
        >>> stmr.stem('fanciful')
        'fancy'
        >>> stmr.stem('torment')
        'tor'


        .. versionadded:: 0.3.0
        .. versionchanged:: 0.3.6
            Encapsulated in class

        """
        terminate = False
        intact = True
        while not terminate:
            for n in range(6, 0, -1):
                if word[-n:] in self._rule_table[n]:
                    accept = False
                    for rule in self._rule_table[n][word[-n:]]:
                        (word, accept, intact, terminate,) = self._apply_rule(
                            word, rule, intact, terminate
                        )
                        if accept:
                            break

                    if accept:
                        break
            else:
                break

        return word


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_chebyshev.

This module contains unit tests for abydos.distance.Chebyshev
"""

import unittest

from abydos.distance import Chebyshev
from abydos.tokenizer import QGrams, WhitespaceTokenizer

from .. import NONQ_FROM, NONQ_TO


class ChebyshevTestCases(unittest.TestCase):
    """Test Chebyshev functions.

    abydos.distance.Chebyshev
    """

    cmp = Chebyshev()
    cmp_q2 = Chebyshev(tokenizer=QGrams(2))
    cmp_ws = Chebyshev(tokenizer=WhitespaceTokenizer())

    def test_chebyshev_dist_abs(self):
        """Test abydos.distance.Chebyshev.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('nelson', ''), 1)
        self.assertEqual(self.cmp.dist_abs('', 'neilsen'), 1)
        self.assertEqual(self.cmp.dist_abs('nelson', 'neilsen'), 1)

        self.assertEqual(self.cmp_q2.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_q2.dist_abs('nelson', ''), 1)
        self.assertEqual(self.cmp_q2.dist_abs('', 'neilsen'), 1)
        self.assertAlmostEqual(self.cmp_q2.dist_abs('nelson', 'neilsen'), 1)

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            1,
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            1,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_ws.dist_abs('the quick', ''), 1)
        self.assertEqual(self.cmp_ws.dist_abs('', 'the quick'), 1)
        self.assertAlmostEqual(self.cmp_ws.dist_abs(NONQ_FROM, NONQ_TO), 1)
        self.assertAlmostEqual(self.cmp_ws.dist_abs(NONQ_TO, NONQ_FROM), 1)

    def test_chebyshev_dist(self):
        """Test abydos.distance.Chebyshev.dist."""
        self.assertRaises(NotImplementedError, self.cmp.dist)

    def test_chebyshev_sim(self):
        """Test abydos.distance.Chebyshev.sim."""
        self.assertRaises(NotImplementedError, self.cmp.sim)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_millar.

This module contains unit tests for abydos.distance.Millar
"""

import unittest

from abydos.distance import Millar


class MillarTestCases(unittest.TestCase):
    """Test Millar functions.

    abydos.distance.Millar
    """

    cmp = Millar()

    def test_millar_dist_abs(self):
        """Test abydos.distance.Millar.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0.0)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1.3862943611198906)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1.3862943611198906)
        self.assertEqual(self.cmp.dist_abs('a', 'a'), 0.0)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 2.772588722239781)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 2.772588722239781)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist_abs('abcd', 'efgh'), 6.931471805599453)

        self.assertAlmostEqual(
            self.cmp.dist_abs('Nigel', 'Niall'), 4.1588830833596715
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Niall', 'Nigel'), 4.1588830833596715
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Colin', 'Coiln'), 4.1588830833596715
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Coiln', 'Colin'), 4.1588830833596715
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('ATCAACGAGT', 'AACGATTAG'), 4.852030263919617
        )

    def test_millar_dist(self):
        """Test abydos.distance.Millar.dist."""
        self.assertRaises(NotImplementedError, self.cmp.dist)

    def test_millar_sim(self):
        """Test abydos.distance.Millar.sim."""
        self.assertRaises(NotImplementedError, self.cmp.sim)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_manhattan.

This module contains unit tests for abydos.distance.Manhattan
"""

import unittest

from abydos.distance import Manhattan
from abydos.tokenizer import QGrams, WhitespaceTokenizer

from .. import NONQ_FROM, NONQ_TO


class ManhattanTestCases(unittest.TestCase):
    """Test Manhattan functions.

    abydos.distance.Manhattan
    """

    cmp = Manhattan()
    cmp_q2 = Manhattan(tokenizer=QGrams(2))
    cmp_ws = Manhattan(tokenizer=WhitespaceTokenizer())

    def test_manhattan_dist_abs(self):
        """Test abydos.distance.Manhattan.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('nelson', ''), 7)
        self.assertEqual(self.cmp.dist_abs('', 'neilsen'), 8)
        self.assertAlmostEqual(self.cmp.dist_abs('nelson', 'neilsen'), 7)

        self.assertEqual(self.cmp_q2.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_q2.dist_abs('nelson', ''), 7)
        self.assertEqual(self.cmp_q2.dist_abs('', 'neilsen'), 8)
        self.assertAlmostEqual(self.cmp_q2.dist_abs('nelson', 'neilsen'), 7)

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            7,
        )
        self.assertEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            8,
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            7,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_ws.dist_abs('the quick', ''), 2)
        self.assertEqual(self.cmp_ws.dist_abs('', 'the quick'), 2)
        self.assertAlmostEqual(self.cmp_ws.dist_abs(NONQ_FROM, NONQ_TO), 8)
        self.assertAlmostEqual(self.cmp_ws.dist_abs(NONQ_TO, NONQ_FROM), 8)

    def test_manhattan_sim(self):
        """Test abydos.distance.Manhattan.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('nelson', ''), 0)
        self.assertEqual(self.cmp.sim('', 'neilsen'), 0)
        self.assertAlmostEqual(self.cmp.sim('nelson', 'neilsen'), 8 / 15)

        self.assertEqual(self.cmp_q2.sim('', ''), 1)
        self.assertEqual(self.cmp_q2.sim('nelson', ''), 0)
        self.assertEqual(self.cmp_q2.sim('', 'neilsen'), 0)
        self.assertAlmostEqual(self.cmp_q2.sim('nelson', 'neilsen'), 8 / 15)

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.sim(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            0,
        )
        self.assertAlmostEqual(
            self.cmp.sim(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            8 / 15,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.sim('', ''), 1)
        self.assertEqual(self.cmp_ws.sim('the quick', ''), 0)
        self.assertEqual(self.cmp_ws.sim('', 'the quick'), 0)
        self.assertAlmostEqual(self.cmp_ws.sim(NONQ_FROM, NONQ_TO), 1 / 2)
        self.assertAlmostEqual(self.cmp_ws.sim(NONQ_TO, NONQ_FROM), 1 / 2)

    def test_manhattan_dist(self):
        """Test abydos.distance.Manhattan.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('nelson', ''), 1)
        self.assertEqual(self.cmp.dist('', 'neilsen'), 1)
        self.assertAlmostEqual(self.cmp.dist('nelson', 'neilsen'), 7 / 15)

        self.assertEqual(self.cmp_q2.dist('', ''), 0)
        self.assertEqual(self.cmp_q2.dist('nelson', ''), 1)
        self.assertEqual(self.cmp_q2.dist('', 'neilsen'), 1)
        self.assertAlmostEqual(self.cmp_q2.dist('nelson', 'neilsen'), 7 / 15)

        # supplied q-gram tests
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            0,
        )
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('').get_counter(),
            ),
            1,
        )
        self.assertEqual(
            self.cmp.dist(
                QGrams().tokenize('').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            1,
        )
        self.assertAlmostEqual(
            self.cmp.dist(
                QGrams().tokenize('nelson').get_counter(),
                QGrams().tokenize('neilsen').get_counter(),
            ),
            7 / 15,
        )

        # non-q-gram tests
        self.assertEqual(self.cmp_ws.dist('', ''), 0)
        self.assertEqual(self.cmp_ws.dist('the quick', ''), 1)
        self.assertEqual(self.cmp_ws.dist('', 'the quick'), 1)
        self.assertAlmostEqual(self.cmp_ws.dist(NONQ_FROM, NONQ_TO), 1 / 2)
        self.assertAlmostEqual(self.cmp_ws.dist(NONQ_TO, NONQ_FROM), 1 / 2)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_nysiis.

This module contains unit tests for abydos.phonetic.NYSIIS
"""

import unittest

from abydos.phonetic import NYSIIS


class NysiisTestCases(unittest.TestCase):
    """Test NYSIIS functions.

    test cases for abydos.phonetic.NYSIIS
    """

    pa = NYSIIS()
    pa_20 = NYSIIS(max_length=20)
    pa_8mod = NYSIIS(max_length=8, modified=True)
    pa_mod = NYSIIS(modified=True)

    def test_nysiis(self):
        """Test abydos.phonetic.NYSIIS."""
        self.assertEqual(self.pa.encode(''), '')

        # http://coryodaniel.com/index.php/2009/12/30/ruby-nysiis-implementation/
        self.assertEqual(self.pa.encode("O'Daniel"), 'ODANAL')
        self.assertEqual(self.pa.encode("O'Donnel"), 'ODANAL')
        self.assertEqual(self.pa.encode('Cory'), 'CARY')
        self.assertEqual(self.pa.encode('Corey'), 'CARY')
        self.assertEqual(self.pa.encode('Kory'), 'CARY')

        # http://ntz-develop.blogspot.com/2011/03/phonetic-algorithms.html
        self.assertEqual(self.pa.encode('Diggell'), 'DAGAL')
        self.assertEqual(self.pa.encode('Dougal'), 'DAGAL')
        self.assertEqual(self.pa.encode('Doughill'), 'DAGAL')
        self.assertEqual(self.pa.encode('Dougill'), 'DAGAL')
        self.assertEqual(self.pa.encode('Dowgill'), 'DAGAL')
        self.assertEqual(self.pa.encode('Dugall'), 'DAGAL')
        self.assertEqual(self.pa.encode('Dugall'), 'DAGAL')
        self.assertEqual(self.pa.encode('Glinde'), 'GLAND')
        self.assertEqual(self.pa_20.encode('Plumridge'), 'PLANRADG')
        self.assertEqual(self.pa.encode('Chinnick'), 'CANAC')
        self.assertEqual(self.pa.encode('Chinnock'), 'CANAC')
        self.assertEqual(self.pa.encode('Chinnock'), 'CANAC')
        self.assertEqual(self.pa.encode('Chomicki'), 'CANAC')
        self.assertEqual(self.pa.encode('Chomicz'), 'CANAC')
        self.assertEqual(self.pa.encode('Schimek'), 'SANAC')
        self.assertEqual(self.pa.encode('Shimuk'), 'SANAC')
        self.assertEqual(self.pa.encode('Simak'), 'SANAC')
        self.assertEqual(self.pa.encode('Simek'), 'SANAC')
        self.assertEqual(self.pa.encode('Simic'), 'SANAC')
        self.assertEqual(self.pa.encode('Sinnock'), 'SANAC')
        self.assertEqual(self.pa.encode('Sinnocke'), 'SANAC')
        self.assertEqual(self.pa.encode('Sunnex'), 'SANAX')
        self.assertEqual(self.pa.encode('Sunnucks'), 'SANAC')
        self.assertEqual(self.pa.encode('Sunock'), 'SANAC')
        self.assertEqual(self.pa_20.encode('Webberley'), 'WABARLY')
        self.assertEqual(self.pa_20.encode('Wibberley'), 'WABARLY')

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('Alpharades'), 'ALFARA')
        self.assertEqual(self.pa.encode('Aschenputtel'), 'ASANPA')
        self.assertEqual(self.pa.encode('Beverly'), 'BAFARL')
        self.assertEqual(self.pa.encode('Hardt'), 'HARD')
        self.assertEqual(self.pa.encode('acknowledge'), 'ACNALA')
        self.assertEqual(self.pa.encode('MacNeill'), 'MCNAL')
        self.assertEqual(self.pa.encode('MacNeill'), self.pa.encode('McNeill'))
        self.assertEqual(self.pa.encode('Knight'), 'NAGT')
        self.assertEqual(self.pa.encode('Knight'), self.pa.encode('Night'))
        self.assertEqual(self.pa.encode('Pfarr'), 'FAR')
        self.assertEqual(self.pa.encode('Phair'), 'FAR')
        self.assertEqual(self.pa.encode('Phair'), self.pa.encode('Pfarr'))
        self.assertEqual(self.pa.encode('Cherokee'), 'CARACY')
        self.assertEqual(self.pa.encode('Iraq'), 'IRAG')

        # max_length bounds tests
        self.assertEqual(NYSIIS(max_length=-1).encode('Niall'), 'NAL')
        self.assertEqual(NYSIIS(max_length=0).encode('Niall'), 'NAL')

    def test_modified_nysiis(self):
        """Test abydos.phonetic.NYSIIS (modified version)."""
        self.assertEqual(NYSIIS(max_length=-1, modified=True).encode(''), '')

        # https://naldc.nal.usda.gov/download/27833/PDF
        # Some of these were... wrong... and have been corrected
        self.assertEqual(self.pa_8mod.encode('Daves'), 'DAV')
        self.assertEqual(self.pa_8mod.encode('Davies'), 'DAVY')
        self.assertEqual(self.pa_8mod.encode('Devies'), 'DAFY')
        self.assertEqual(self.pa_8mod.encode('Divish'), 'DAVAS')
        self.assertEqual(self.pa_8mod.encode('Dove'), 'DAV')
        self.assertEqual(self.pa_8mod.encode('Devese'), 'DAFAS')
        self.assertEqual(self.pa_8mod.encode('Devies'), 'DAFY')
        self.assertEqual(self.pa_8mod.encode('Devos'), 'DAF')

        self.assertEqual(self.pa_8mod.encode('Schmit'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Schmitt'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Schmitz'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Schmoutz'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Schnitt'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smit'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smite'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smits'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smoot'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smuts'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Sneath'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smyth'), 'SNAT')
        self.assertEqual(self.pa_8mod.encode('Smithy'), 'SNATY')
        self.assertEqual(self.pa_8mod.encode('Smithey'), 'SNATY')

        # http://www.dropby.com/NYSIISTextStrings.html
        # Some of these have been altered since the above uses a different set
        # of modifications.
        self.assertEqual(self.pa_8mod.encode('Edwards'), 'EDWAD')
        self.assertEqual(self.pa_8mod.encode('Perez'), 'PAR')
        self.assertEqual(self.pa_8mod.encode('Macintosh'), 'MCANTAS')
        self.assertEqual(self.pa_8mod.encode('Phillipson'), 'FALAPSAN')
        self.assertEqual(self.pa_8mod.encode('Haddix'), 'HADAC')
        self.assertEqual(self.pa_8mod.encode('Essex'), 'ESAC')
        self.assertEqual(self.pa_8mod.encode('Moye'), 'MY')
        self.assertEqual(self.pa_8mod.encode('McKee'), 'MCY')
        self.assertEqual(self.pa_8mod.encode('Mackie'), 'MCY')
        self.assertEqual(self.pa_8mod.encode('Heitschmidt'), 'HATSNAD')
        self.assertEqual(self.pa_8mod.encode('Bart'), 'BAD')
        self.assertEqual(self.pa_8mod.encode('Hurd'), 'HAD')
        self.assertEqual(self.pa_8mod.encode('Hunt'), 'HAN')
        self.assertEqual(self.pa_8mod.encode('Westerlund'), 'WASTARLA')
        self.assertEqual(self.pa_8mod.encode('Evers'), 'EVAR')
        self.assertEqual(self.pa_8mod.encode('Devito'), 'DAFAT')
        self.assertEqual(self.pa_8mod.encode('Rawson'), 'RASAN')
        self.assertEqual(self.pa_8mod.encode('Shoulders'), 'SALDAR')
        self.assertEqual(self.pa_8mod.encode('Leighton'), 'LATAN')
        self.assertEqual(self.pa_8mod.encode('Wooldridge'), 'WALDRAG')
        self.assertEqual(self.pa_8mod.encode('Oliphant'), 'OLAFAN')
        self.assertEqual(self.pa_8mod.encode('Hatchett'), 'HATCAT')
        self.assertEqual(self.pa_8mod.encode('McKnight'), 'MCNAT')
        self.assertEqual(self.pa_8mod.encode('Rickert'), 'RACAD')
        self.assertEqual(self.pa_8mod.encode('Bowman'), 'BANAN')
        self.assertEqual(self.pa_8mod.encode('Vasquez'), 'VASG')
        self.assertEqual(self.pa_8mod.encode('Bashaw'), 'BAS')
        self.assertEqual(self.pa_8mod.encode('Schoenhoeft'), 'SANAFT')
        self.assertEqual(self.pa_8mod.encode('Heywood'), 'HAD')
        self.assertEqual(self.pa_8mod.encode('Hayman'), 'HANAN')
        self.assertEqual(self.pa_8mod.encode('Seawright'), 'SARAT')
        self.assertEqual(self.pa_8mod.encode('Kratzer'), 'CRATSAR')
        self.assertEqual(self.pa_8mod.encode('Canaday'), 'CANADY')
        self.assertEqual(self.pa_8mod.encode('Crepeau'), 'CRAP')

        # Additional tests from @Yomguithereal's talisman
        # https://github.com/Yomguithereal/talisman/blob/master/test/phonetics/nysiis.js
        self.assertEqual(self.pa_8mod.encode('Andrew'), 'ANDR')
        self.assertEqual(self.pa_8mod.encode('Robertson'), 'RABARTSA')
        self.assertEqual(self.pa_8mod.encode('Nolan'), 'NALAN')
        self.assertEqual(self.pa_8mod.encode('Louis XVI'), 'LASXV')
        self.assertEqual(self.pa_8mod.encode('Case'), 'CAS')
        self.assertEqual(self.pa_8mod.encode('Mclaughlin'), 'MCLAGLAN')
        self.assertEqual(self.pa_8mod.encode('Awale'), 'AL')
        self.assertEqual(self.pa_8mod.encode('Aegir'), 'AGAR')
        self.assertEqual(self.pa_8mod.encode('Lundgren'), 'LANGRAN')
        self.assertEqual(self.pa_8mod.encode('Philbert'), 'FALBAD')
        self.assertEqual(self.pa_8mod.encode('Harry'), 'HARY')
        self.assertEqual(self.pa_8mod.encode('Mackenzie'), 'MCANSY')

        # max_length bounds tests
        self.assertEqual(
            NYSIIS(max_length=-1, modified=True).encode('Niall'), 'NAL'
        )
        self.assertEqual(
            NYSIIS(max_length=0, modified=True).encode('Niall'), 'NAL'
        )

        # coverage
        self.assertEqual(self.pa_mod.encode('Sam Jr.'), 'ERROR')
        self.assertEqual(self.pa_mod.encode('John Sr.'), 'ERROR')
        self.assertEqual(self.pa_mod.encode('Wright'), 'RAT')
        self.assertEqual(self.pa_mod.encode('Rhodes'), 'RAD')
        self.assertEqual(self.pa_mod.encode('Dgagoda'), 'GAGAD')
        self.assertEqual(self.pa_mod.encode('Bosch'), 'BAS')
        self.assertEqual(self.pa_mod.encode('Schrader'), 'SRADAR')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_needleman_wunsch.

This module contains unit tests for abydos.distance.NeedlemanWunsch
"""

import unittest

from abydos.distance import NeedlemanWunsch

from .. import NIALL


def _sim_wikipedia(src, tar):
    """Return a similarity score for two DNA base pairs.

    Values copied from:
    https://en.wikipedia.org/wiki/Needleman%E2%80%93Wunsch_algorithm

    Parameters
    ----------
    src : str
        Source string for comparison
        tar : str
        Target string for comparison

    Returns
    -------
    int: similarity of two DNA base pairs

    """
    nw_matrix = {
        ('A', 'A'): 10,
        ('G', 'G'): 7,
        ('C', 'C'): 9,
        ('T', 'T'): 8,
        ('A', 'G'): -1,
        ('A', 'C'): -3,
        ('A', 'T'): -4,
        ('G', 'C'): -5,
        ('G', 'T'): -3,
        ('C', 'T'): 0,
    }
    return NeedlemanWunsch.sim_matrix(
        src, tar, nw_matrix, symmetric=True, alphabet='CGAT'
    )


def _sim_nw(src, tar):
    """Return 1 if src is tar, otherwise -1.

    Parameters
    ----------
    src : str
        Source string for comparison
        tar : str
        Target string for comparison

    Returns
    -------
    int: nw similarity

    """
    return 2 * int(src is tar) - 1


class MatrixSimTestCases(unittest.TestCase):
    """Test matrix similarity functions.

    abydos.distance.NeedlemanWunsch.sim_matrix
    """

    def test_sim_matrix(self):
        """Test abydos.distance.NeedlemanWunsch.sim_matrix."""
        self.assertEqual(NeedlemanWunsch.sim_matrix('', ''), 1)
        self.assertEqual(NeedlemanWunsch.sim_matrix('', 'a'), 0)
        self.assertEqual(NeedlemanWunsch.sim_matrix('a', ''), 0)
        self.assertEqual(NeedlemanWunsch.sim_matrix('a', 'a'), 1)
        self.assertEqual(NeedlemanWunsch.sim_matrix('abcd', 'abcd'), 1)
        self.assertEqual(NeedlemanWunsch.sim_matrix('abcd', 'dcba'), 0)
        self.assertEqual(NeedlemanWunsch.sim_matrix('abc', 'cba'), 0)

        # https://en.wikipedia.org/wiki/Needleman–Wunsch_algorithm
        self.assertEqual(_sim_wikipedia('A', 'C'), -3)
        self.assertEqual(_sim_wikipedia('G', 'G'), 7)
        self.assertEqual(_sim_wikipedia('A', 'A'), 10)
        self.assertEqual(_sim_wikipedia('T', 'A'), -4)
        self.assertEqual(_sim_wikipedia('T', 'C'), 0)
        self.assertEqual(_sim_wikipedia('A', 'G'), -1)
        self.assertEqual(_sim_wikipedia('C', 'T'), 0)

        self.assertRaises(
            ValueError, NeedlemanWunsch.sim_matrix, 'abc', 'cba', alphabet='ab'
        )
        self.assertRaises(
            ValueError, NeedlemanWunsch.sim_matrix, 'abc', 'ba', alphabet='ab'
        )
        self.assertRaises(
            ValueError, NeedlemanWunsch.sim_matrix, 'ab', 'cba', alphabet='ab'
        )


class NeedlemanWunschTestCases(unittest.TestCase):
    """Test Needleman-Wunsch functions.

    abydos.distance.NeedlemanWunsch
    """

    def test_needleman_wunsch_sim_score(self):
        """Test abydos.distance.NeedlemanWunsch.sim_score."""
        self.assertEqual(NeedlemanWunsch().sim_score('', ''), 0)

        # https://en.wikipedia.org/wiki/Needleman–Wunsch_algorithm
        self.assertEqual(
            NeedlemanWunsch(1, _sim_nw).sim_score('GATTACA', 'GCATGCU'), 0
        )
        self.assertEqual(
            NeedlemanWunsch(5, _sim_wikipedia).sim_score(
                'AGACTAGTTAC', 'CGAGACGT'
            ),
            16,
        )

        # checked against http://ds9a.nl/nwunsch/ (mismatch=1, gap=5, skew=5)
        nw5 = NeedlemanWunsch(5, _sim_nw)
        self.assertEqual(nw5.sim_score('CGATATCAG', 'TGACGSTGC'), -5)
        self.assertEqual(nw5.sim_score('AGACTAGTTAC', 'TGACGSTGC'), -7)
        self.assertEqual(nw5.sim_score('AGACTAGTTAC', 'CGAGACGT'), -15)

    def test_needleman_wunsch_dist_abs_nialls(self):
        """Test abydos.distance.NeedlemanWunsch.dist_abs (Nialls set)."""
        # checked against http://ds9a.nl/nwunsch/ (mismatch=1, gap=2, skew=2)
        nw_vals = (5, 0, -2, 3, 1, 1, -2, -2, -1, -3, -3, -5, -3, -7, -7, -19)
        nw2 = NeedlemanWunsch(2, _sim_nw)
        for i in range(len(NIALL)):
            self.assertEqual(nw2.sim_score(NIALL[0], NIALL[i]), nw_vals[i])

    def test_needleman_wunsch_sim(self):
        """Test abydos.distance.NeedlemanWunsch.sim."""
        self.assertEqual(NeedlemanWunsch().sim('', ''), 1.0)

        # https://en.wikipedia.org/wiki/Needleman–Wunsch_algorithm
        self.assertEqual(
            NeedlemanWunsch(1, _sim_nw).sim('GATTACA', 'GCATGCU'), 0
        )
        self.assertEqual(
            NeedlemanWunsch(5, _sim_wikipedia).sim('AGACTAGTTAC', 'CGAGACGT'),
            0.19950186722152657,
        )

        # checked against http://ds9a.nl/nwunsch/ (mismatch=1, gap=5, skew=5)
        nw5 = NeedlemanWunsch(5, _sim_nw)
        self.assertEqual(nw5.sim('CGATATCAG', 'TGACGSTGC'), 0)
        self.assertEqual(nw5.sim('AGACTAGTTAC', 'TGACGSTGC'), 0)
        self.assertEqual(nw5.sim('AGACTAGTTAC', 'CGAGACGT'), 0)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_bwtf.

This module contains unit tests for abydos.fingerprint.BWTF
"""

import unittest

from abydos.fingerprint import BWTF


class BWTFTestCases(unittest.TestCase):
    """Test BWT fingerprint.

    abydos.fingerprint.BWTF
    """

    bwt = BWTF()
    bwt_pipe = BWTF('|')
    bwt_dollar = BWTF('$')

    def test_consonant_bwtf(self):
        """Test abydos.fingerprint.BWTF."""
        # Examples from Wikipedia entry on BWT
        self.assertEqual(self.bwt.fingerprint(''), '\x00')
        self.assertEqual(self.bwt_pipe.fingerprint('^BANANA'), 'BNN^AA|A')
        self.assertEqual(
            self.bwt_pipe.fingerprint(
                'SIX.MIXED.PIXIES.SIFT.SIXTY.PIXIE.DUST.BOXES'
            ),
            'TEXYDST.E.IXIXIXXSSMPPS.B..E.|.UESFXDIIOIIITS',
        )

        self.assertEqual(self.bwt_dollar.fingerprint('aardvark'), 'k$avrraad')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_pshp_soundex_first.

This module contains unit tests for abydos.phonetic.PSHPSoundexFirst
"""

import unittest

from abydos.phonetic import PSHPSoundexFirst


class PSHPSoundexTestCases(unittest.TestCase):
    """Test PSHP Soundex functions.

    test cases for abydos.phonetic.PSHPSoundexFirst
    """

    pa = PSHPSoundexFirst()
    pa_german = PSHPSoundexFirst(german=True)
    pa_unl = PSHPSoundexFirst(max_length=-1)

    def test_pshp_soundex_first(self):
        """Test abydos.phonetic.PSHPSoundexFirst."""
        # Base case
        self.assertEqual(self.pa.encode(''), '0000')

        # Examples given in defining paper (Hershberg, et al. 1976)
        self.assertEqual(self.pa.encode('JAMES'), 'J700')
        self.assertEqual(self.pa.encode('JOHN'), 'J500')
        self.assertEqual(self.pa.encode('PAT'), 'P700')
        self.assertEqual(self.pa.encode('PETER'), 'P300')

        # Additions for coverage
        self.assertEqual(self.pa.encode('Giles'), 'J400')
        self.assertEqual(self.pa.encode('Cy'), 'S000')
        self.assertEqual(self.pa.encode('Chris'), 'K500')
        self.assertEqual(self.pa.encode('Caleb'), 'K400')
        self.assertEqual(self.pa.encode('Knabe'), 'N100')
        self.assertEqual(self.pa.encode('Phil'), 'F400')
        self.assertEqual(self.pa.encode('Wieland'), 'V400')
        self.assertEqual(self.pa_german.encode('Wayne'), 'V500')
        self.assertEqual(self.pa_unl.encode('Christopher'), 'K5')
        self.assertEqual(self.pa_unl.encode('Asdaananndsjsjasd'), 'A23553223')
        self.assertEqual(self.pa.encode('Asdaananndsjsjasd'), 'A235')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('JAMES'), 'JN')
        self.assertEqual(self.pa.encode_alpha('JOHN'), 'JN')
        self.assertEqual(self.pa.encode_alpha('PAT'), 'PT')
        self.assertEqual(self.pa.encode_alpha('PETER'), 'PT')
        self.assertEqual(self.pa.encode_alpha('Knabe'), 'NP')
        self.assertEqual(self.pa.encode_alpha('Phil'), 'FL')
        self.assertEqual(self.pa.encode_alpha('Wieland'), 'VL')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._baroni_urbani_buser_ii.

Baroni-Urbani & Buser II correlation
"""

from typing import Any, Counter as TCounter, Optional, Sequence, Set, Union

from ._token_distance import _TokenDistance
from ..tokenizer import _Tokenizer

__all__ = ['BaroniUrbaniBuserII']


class BaroniUrbaniBuserII(_TokenDistance):
    r"""Baroni-Urbani & Buser II correlation.

    For two sets X and Y and a population N, the Baroni-Urbani & Buser II
    correlation :cite:`BaroniUrbani:1976` is

        .. math::

            corr_{BaroniUrbaniBuserII}(X, Y) =
            \frac{\sqrt{|X \cap Y| \cdot |(N \setminus X) \setminus Y|} +
            |X \cap Y| - |X \setminus Y| - |Y \setminus X|}
            {\sqrt{|X \cap Y| \cdot |(N \setminus X) \setminus Y|} +
            |X \cap Y| + |X \setminus Y| + |Y \setminus X|}

    This is the first, but less commonly used and referenced of the two
    similarities proposed by Baroni-Urbani & Buser.

    In :ref:`2x2 confusion table terms <confusion_table>`, where a+b+c+d=n,
    this is

        .. math::

            corr_{BaroniUrbaniBuserII} =
            \frac{\sqrt{ad}+a-b-c}{\sqrt{ad}+a+b+c}

    .. versionadded:: 0.4.0
    """

    def __init__(
        self,
        alphabet: Optional[
            Union[TCounter[str], Sequence[str], Set[str], int]
        ] = None,
        tokenizer: Optional[_Tokenizer] = None,
        intersection_type: str = 'crisp',
        **kwargs: Any
    ) -> None:
        """Initialize BaroniUrbaniBuserII instance.

        Parameters
        ----------
        alphabet : Counter, collection, int, or None
            This represents the alphabet of possible tokens.
            See :ref:`alphabet <alphabet>` description in
            :py:class:`_TokenDistance` for details.
        tokenizer : _Tokenizer
            A tokenizer instance from the :py:mod:`abydos.tokenizer` package
        intersection_type : str
            Specifies the intersection type, and set type as a result:
            See :ref:`intersection_type <intersection_type>` description in
            :py:class:`_TokenDistance` for details.
        **kwargs
            Arbitrary keyword arguments

        Other Parameters
        ----------------
        qval : int
            The length of each q-gram. Using this parameter and tokenizer=None
            will cause the instance to use the QGram tokenizer with this
            q value.
        metric : _Distance
            A string distance measure class for use in the ``soft`` and
            ``fuzzy`` variants.
        threshold : float
            A threshold value, similarities above which are counted as
            members of the intersection for the ``fuzzy`` variant.


        .. versionadded:: 0.4.0

        """
        super(BaroniUrbaniBuserII, self).__init__(
            alphabet=alphabet,
            tokenizer=tokenizer,
            intersection_type=intersection_type,
            **kwargs
        )

    def corr(self, src: str, tar: str) -> float:
        """Return the Baroni-Urbani & Buser II correlation of two strings.

        Parameters
        ----------
        src : str
            Source string (or QGrams/Counter objects) for comparison
        tar : str
            Target string (or QGrams/Counter objects) for comparison

        Returns
        -------
        float
            Baroni-Urbani & Buser II correlation

        Examples
        --------
        >>> cmp = BaroniUrbaniBuserII()
        >>> cmp.corr('cat', 'hat')
        0.8239675481756209
        >>> cmp.corr('Niall', 'Neil')
        0.7105646350028408
        >>> cmp.corr('aluminum', 'Catalan')
        0.31398542410970204
        >>> cmp.corr('ATCG', 'TAGC')
        -1.0


        .. versionadded:: 0.4.0

        """
        if src == tar:
            return 1.0

        self._tokenize(src, tar)

        a = self._intersection_card()
        b = self._src_only_card()
        c = self._tar_only_card()
        d = self._total_complement_card()

        return ((a * d) ** 0.5 + a - b - c) / ((a * d) ** 0.5 + a + b + c)

    def sim(self, src: str, tar: str) -> float:
        """Return the Baroni-Urbani & Buser II similarity of two strings.

        Parameters
        ----------
        src : str
            Source string (or QGrams/Counter objects) for comparison
        tar : str
            Target string (or QGrams/Counter objects) for comparison

        Returns
        -------
        float
            Baroni-Urbani & Buser II similarity

        Examples
        --------
        >>> cmp = BaroniUrbaniBuserII()
        >>> cmp.sim('cat', 'hat')
        0.9119837740878105
        >>> cmp.sim('Niall', 'Neil')
        0.8552823175014204
        >>> cmp.sim('aluminum', 'Catalan')
        0.656992712054851
        >>> cmp.sim('ATCG', 'TAGC')
        0.0


        .. versionadded:: 0.4.0

        """
        return (self.corr(src, tar) + 1) / 2


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_refined_soundex.

This module contains unit tests for abydos.phonetic.RefinedSoundex
"""

import unittest

from abydos.phonetic import RefinedSoundex


class RefinedSoundexTestCases(unittest.TestCase):
    """Test Refined Soundex functions.

    test cases for abydos.phonetic.RefinedSoundex
    """

    pa = RefinedSoundex()

    def test_refined_soundex(self):
        """Test abydos.phonetic.RefinedSoundex."""
        # http://ntz-develop.blogspot.com/2011/03/phonetic-algorithms.html
        self.assertEqual(self.pa.encode('Braz'), 'B95')
        self.assertEqual(self.pa.encode('Broz'), 'B95')
        self.assertEqual(self.pa.encode('Caren'), 'C98')
        self.assertEqual(self.pa.encode('Caron'), 'C98')
        self.assertEqual(self.pa.encode('Carren'), 'C98')
        self.assertEqual(self.pa.encode('Charon'), 'C98')
        self.assertEqual(self.pa.encode('Corain'), 'C98')
        self.assertEqual(self.pa.encode('Coram'), 'C98')
        self.assertEqual(self.pa.encode('Corran'), 'C98')
        self.assertEqual(self.pa.encode('Corrin'), 'C98')
        self.assertEqual(self.pa.encode('Corwin'), 'C98')
        self.assertEqual(self.pa.encode('Curran'), 'C98')
        self.assertEqual(self.pa.encode('Curreen'), 'C98')
        self.assertEqual(self.pa.encode('Currin'), 'C98')
        self.assertEqual(self.pa.encode('Currom'), 'C98')
        self.assertEqual(self.pa.encode('Currum'), 'C98')
        self.assertEqual(self.pa.encode('Curwen'), 'C98')
        self.assertEqual(self.pa.encode('Hairs'), 'H93')
        self.assertEqual(self.pa.encode('Hark'), 'H93')
        self.assertEqual(self.pa.encode('Hars'), 'H93')
        self.assertEqual(self.pa.encode('Hayers'), 'H93')
        self.assertEqual(self.pa.encode('Heers'), 'H93')
        self.assertEqual(self.pa.encode('Hiers'), 'H93')
        self.assertEqual(self.pa.encode('Lambard'), 'L8196')
        self.assertEqual(self.pa.encode('Lambart'), 'L8196')
        self.assertEqual(self.pa.encode('Lambert'), 'L8196')
        self.assertEqual(self.pa.encode('Lambird'), 'L8196')
        self.assertEqual(self.pa.encode('Lampaert'), 'L8196')
        self.assertEqual(self.pa.encode('Lampard'), 'L8196')
        self.assertEqual(self.pa.encode('Lampart'), 'L8196')
        self.assertEqual(self.pa.encode('Lamperd'), 'L8196')
        self.assertEqual(self.pa.encode('Lampert'), 'L8196')
        self.assertEqual(self.pa.encode('Lamport'), 'L8196')
        self.assertEqual(self.pa.encode('Limbert'), 'L8196')
        self.assertEqual(self.pa.encode('Lombard'), 'L8196')
        self.assertEqual(self.pa.encode('Nolton'), 'N768')
        self.assertEqual(self.pa.encode('Noulton'), 'N768')

        # http://trimc-nlp.blogspot.com/2015/03/the-soundex-algorithm.html
        self.assertEqual(self.pa.encode('Craig'), 'C94')
        self.assertEqual(self.pa.encode('Crag'), 'C94')
        self.assertEqual(self.pa.encode('Crejg'), 'C94')
        self.assertEqual(self.pa.encode('Creig'), 'C94')
        self.assertEqual(self.pa.encode('Craigg'), 'C94')
        self.assertEqual(self.pa.encode('Craug'), 'C94')
        self.assertEqual(self.pa.encode('Craiggg'), 'C94')
        self.assertEqual(self.pa.encode('Creg'), 'C94')
        self.assertEqual(self.pa.encode('Cregg'), 'C94')
        self.assertEqual(self.pa.encode('Creag'), 'C94')
        self.assertEqual(self.pa.encode('Greg'), 'G94')
        self.assertEqual(self.pa.encode('Gregg'), 'G94')
        self.assertEqual(self.pa.encode('Graig'), 'G94')
        self.assertEqual(self.pa.encode('Greig'), 'G94')
        self.assertEqual(self.pa.encode('Greggg'), 'G94')
        self.assertEqual(self.pa.encode('Groeg'), 'G94')
        self.assertEqual(self.pa.encode('Graj'), 'G94')
        self.assertEqual(self.pa.encode('Grej'), 'G94')
        self.assertEqual(self.pa.encode('Grreg'), 'G94')
        self.assertEqual(self.pa.encode('Greag'), 'G94')
        self.assertEqual(self.pa.encode('Grig'), 'G94')
        self.assertEqual(self.pa.encode('Kregg'), 'K94')
        self.assertEqual(self.pa.encode('Kraig'), 'K94')
        self.assertEqual(self.pa.encode('Krag'), 'K94')
        self.assertEqual(self.pa.encode('Kreig'), 'K94')
        self.assertEqual(self.pa.encode('Krug'), 'K94')
        self.assertEqual(self.pa.encode('Kreg'), 'K94')
        self.assertEqual(self.pa.encode('Krieg'), 'K94')
        self.assertEqual(self.pa.encode('Krijg'), 'K94')

        # Apache Commons test cases
        # http://svn.apache.org/viewvc/commons/proper/codec/trunk/src/test/java/org/apache/commons/codec/language/RefinedSoundexTest.java?view=markup
        self.assertEqual(self.pa.encode('testing'), 'T3684')
        self.assertEqual(self.pa.encode('TESTING'), 'T3684')
        self.assertEqual(self.pa.encode('The'), 'T')
        self.assertEqual(self.pa.encode('quick'), 'Q3')
        self.assertEqual(self.pa.encode('brown'), 'B98')
        self.assertEqual(self.pa.encode('fox'), 'F5')
        self.assertEqual(self.pa.encode('jumped'), 'J816')
        self.assertEqual(self.pa.encode('over'), 'O29')
        self.assertEqual(self.pa.encode('the'), 'T')
        self.assertEqual(self.pa.encode('lazy'), 'L5')
        self.assertEqual(self.pa.encode('dogs'), 'D43')

        # Test with retain_vowels=True
        # http://ntz-develop.blogspot.com/2011/03/phonetic-algorithms.html
        pa_vowels = RefinedSoundex(retain_vowels=True)
        self.assertEqual(pa_vowels.encode('Braz'), 'B905')
        self.assertEqual(pa_vowels.encode('Broz'), 'B905')
        self.assertEqual(pa_vowels.encode('Caren'), 'C0908')
        self.assertEqual(pa_vowels.encode('Caron'), 'C0908')
        self.assertEqual(pa_vowels.encode('Carren'), 'C0908')
        self.assertEqual(pa_vowels.encode('Charon'), 'C0908')
        self.assertEqual(pa_vowels.encode('Corain'), 'C0908')
        self.assertEqual(pa_vowels.encode('Coram'), 'C0908')
        self.assertEqual(pa_vowels.encode('Corran'), 'C0908')
        self.assertEqual(pa_vowels.encode('Corrin'), 'C0908')
        self.assertEqual(pa_vowels.encode('Corwin'), 'C0908')
        self.assertEqual(pa_vowels.encode('Curran'), 'C0908')
        self.assertEqual(pa_vowels.encode('Curreen'), 'C0908')
        self.assertEqual(pa_vowels.encode('Currin'), 'C0908')
        self.assertEqual(pa_vowels.encode('Currom'), 'C0908')
        self.assertEqual(pa_vowels.encode('Currum'), 'C0908')
        self.assertEqual(pa_vowels.encode('Curwen'), 'C0908')
        self.assertEqual(pa_vowels.encode('Hairs'), 'H093')
        self.assertEqual(pa_vowels.encode('Hark'), 'H093')
        self.assertEqual(pa_vowels.encode('Hars'), 'H093')
        self.assertEqual(pa_vowels.encode('Hayers'), 'H093')
        self.assertEqual(pa_vowels.encode('Heers'), 'H093')
        self.assertEqual(pa_vowels.encode('Hiers'), 'H093')
        self.assertEqual(pa_vowels.encode('Lambard'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lambart'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lambert'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lambird'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lampaert'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lampard'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lampart'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lamperd'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lampert'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lamport'), 'L081096')
        self.assertEqual(pa_vowels.encode('Limbert'), 'L081096')
        self.assertEqual(pa_vowels.encode('Lombard'), 'L081096')
        self.assertEqual(pa_vowels.encode('Nolton'), 'N07608')
        self.assertEqual(pa_vowels.encode('Noulton'), 'N07608')

        # http://trimc-nlp.blogspot.com/2015/03/the-soundex-algorithm.html
        self.assertEqual(pa_vowels.encode('Craig'), 'C904')
        self.assertEqual(pa_vowels.encode('Crag'), 'C904')
        self.assertEqual(pa_vowels.encode('Crejg'), 'C904')
        self.assertEqual(pa_vowels.encode('Creig'), 'C904')
        self.assertEqual(pa_vowels.encode('Craigg'), 'C904')
        self.assertEqual(pa_vowels.encode('Craug'), 'C904')
        self.assertEqual(pa_vowels.encode('Craiggg'), 'C904')
        self.assertEqual(pa_vowels.encode('Creg'), 'C904')
        self.assertEqual(pa_vowels.encode('Cregg'), 'C904')
        self.assertEqual(pa_vowels.encode('Creag'), 'C904')
        self.assertEqual(pa_vowels.encode('Greg'), 'G904')
        self.assertEqual(pa_vowels.encode('Gregg'), 'G904')
        self.assertEqual(pa_vowels.encode('Graig'), 'G904')
        self.assertEqual(pa_vowels.encode('Greig'), 'G904')
        self.assertEqual(pa_vowels.encode('Greggg'), 'G904')
        self.assertEqual(pa_vowels.encode('Groeg'), 'G904')
        self.assertEqual(pa_vowels.encode('Graj'), 'G904')
        self.assertEqual(pa_vowels.encode('Grej'), 'G904')
        self.assertEqual(pa_vowels.encode('Grreg'), 'G904')
        self.assertEqual(pa_vowels.encode('Greag'), 'G904')
        self.assertEqual(pa_vowels.encode('Grig'), 'G904')
        self.assertEqual(pa_vowels.encode('Kregg'), 'K904')
        self.assertEqual(pa_vowels.encode('Kraig'), 'K904')
        self.assertEqual(pa_vowels.encode('Krag'), 'K904')
        self.assertEqual(pa_vowels.encode('Kreig'), 'K904')
        self.assertEqual(pa_vowels.encode('Krug'), 'K904')
        self.assertEqual(pa_vowels.encode('Kreg'), 'K904')
        self.assertEqual(pa_vowels.encode('Krieg'), 'K904')
        self.assertEqual(pa_vowels.encode('Krijg'), 'K904')

        # Apache Commons test cases
        # http://svn.apache.org/viewvc/commons/proper/codec/trunk/src/test/java/org/apache/commons/codec/language/RefinedSoundexTest.java?view=markup
        self.assertEqual(pa_vowels.encode('testing'), 'T036084')
        self.assertEqual(pa_vowels.encode('TESTING'), 'T036084')
        self.assertEqual(pa_vowels.encode('The'), 'T0')
        self.assertEqual(pa_vowels.encode('quick'), 'Q03')
        self.assertEqual(pa_vowels.encode('brown'), 'B908')
        self.assertEqual(pa_vowels.encode('fox'), 'F05')
        self.assertEqual(pa_vowels.encode('jumped'), 'J08106')
        self.assertEqual(pa_vowels.encode('over'), 'O209')
        self.assertEqual(pa_vowels.encode('the'), 'T0')
        self.assertEqual(pa_vowels.encode('lazy'), 'L050')
        self.assertEqual(pa_vowels.encode('dogs'), 'D043')

        # length tests
        pa_40 = RefinedSoundex(max_length=4, zero_pad=True)
        self.assertEqual(pa_40.encode('testing'), 'T368')
        self.assertEqual(pa_40.encode('TESTING'), 'T368')
        self.assertEqual(pa_40.encode('The'), 'T000')
        self.assertEqual(pa_40.encode('quick'), 'Q300')
        self.assertEqual(pa_40.encode('brown'), 'B980')
        self.assertEqual(pa_40.encode('fox'), 'F500')
        self.assertEqual(pa_40.encode('jumped'), 'J816')
        self.assertEqual(pa_40.encode('over'), 'O290')
        self.assertEqual(pa_40.encode('the'), 'T000')
        self.assertEqual(pa_40.encode('lazy'), 'L500')
        self.assertEqual(pa_40.encode('dogs'), 'D430')
        pa_4 = RefinedSoundex(max_length=4)
        self.assertEqual(pa_4.encode('The'), 'T')
        self.assertEqual(pa_4.encode('quick'), 'Q3')
        self.assertEqual(pa_4.encode('brown'), 'B98')
        self.assertEqual(pa_4.encode('fox'), 'F5')
        self.assertEqual(pa_4.encode('jumped'), 'J816')
        self.assertEqual(pa_4.encode('over'), 'O29')
        self.assertEqual(pa_4.encode('the'), 'T')
        self.assertEqual(pa_4.encode('lazy'), 'L5')
        self.assertEqual(pa_4.encode('dogs'), 'D43')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Broz'), 'BRZ')
        self.assertEqual(self.pa.encode_alpha('Caren'), 'CRN')
        self.assertEqual(self.pa.encode_alpha('Hairs'), 'HRK')
        self.assertEqual(self.pa.encode_alpha('Lamperd'), 'LNPRT')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_qgrams.

This module contains unit tests for abydos.tokenizer.QGrams
"""

import unittest

from abydos.tokenizer import LegaliPyTokenizer

from .. import _corpus_file


class LegaliPyTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.LegaliPyTokenizer."""

    def test_legalipy_tokenizer(self):
        """Test abydos.tokenizer.LegaliPyTokenizer."""
        try:
            from syllabipy.legalipy import LegaliPy  # noqa: F401
        except ImportError:  # pragma: no cover
            return

        self.assertEqual(
            sorted(LegaliPyTokenizer().tokenize('').get_list()), ['']
        )
        self.assertEqual(
            sorted(LegaliPyTokenizer().tokenize('a').get_list()), ['a']
        )

        self.assertEqual(
            sorted(LegaliPyTokenizer().tokenize('nelson').get_list()),
            sorted(['n', 'els', 'on']),
        )
        self.assertEqual(
            sorted(LegaliPyTokenizer().tokenize('neilson').get_list()),
            sorted(['n', 'eils', 'on']),
        )

        tok = LegaliPyTokenizer()
        with open(_corpus_file('wikipediaCommonMisspellings.csv')) as corpus:
            text = ' '.join([_.split(',')[1] for _ in corpus.readlines()])
        tok.train_onsets(text)

        with open(_corpus_file('misspellings.csv')) as corpus:
            text = ' '.join([_.split(',')[1] for _ in corpus.readlines()])
        tok.train_onsets(text, append=True)

        self.assertEqual(
            sorted(tok.tokenize('nelson').get_list()), sorted(['nel', 'son'])
        )
        self.assertEqual(
            sorted(tok.tokenize('neilson').get_list()),
            sorted(['ne', 'il', 'son']),
        )
        self.assertEqual(
            sorted(tok.tokenize('peninsular').get_list()),
            sorted(['pe', 'nin', 'su', 'lar']),
        )
        self.assertEqual(
            sorted(tok.tokenize('spectacular').get_list()),
            sorted(['spec', 'ta', 'cu', 'lar']),
        )
        self.assertEqual(
            sorted(tok.tokenize('sufficiently').get_list()),
            sorted(['suf', 'fi', 'ci', 'ent', 'ly']),
        )
        self.assertEqual(
            sorted(tok.tokenize('yachting').get_list()),
            sorted(['y', 'ach', 'ting']),
        )
        self.assertEqual(
            sorted(tok.tokenize('caterpillars').get_list()),
            sorted(['ca', 'ter', 'pil', 'lars']),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_snowball_dutch.

This module contains unit tests for abydos.stemmer.SnowballDutch
"""

import codecs
import unittest

from abydos.stemmer import SnowballDutch

from .. import _corpus_file


class SnowballDutchTestCases(unittest.TestCase):
    """Test Snowball functions.

    abydos.stemmer.SnowballDutch
    """

    stmr = SnowballDutch()

    def test_snowball_dutch(self):
        """Test abydos.stemmer.SnowballDutch (Snowball testset).

        These test cases are from
        http://snowball.tartarus.org/algorithms/dutch/diffs.txt
        """
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        #  Snowball Dutch test set
        with codecs.open(
            _corpus_file('snowball_dutch.csv'), encoding='utf-8'
        ) as snowball_ts:
            next(snowball_ts)
            for line in snowball_ts:
                if line[0] != '#':
                    line = line.strip().split(',')
                    word, stem = line[0], line[1]
                    self.assertEqual(self.stmr.stem(word), stem.lower())

        # missed branch test cases
        self.assertEqual(self.stmr.stem('zondulielijk'), 'zondulie')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_porter.

This module contains unit tests for abydos.stemmer.Porter
"""

import unittest

from abydos.stemmer import Porter


from .. import _corpus_file


class PorterTestCases(unittest.TestCase):
    """Test Porter functions.

    abydos.stemmer.Porter
    """

    stmr = Porter()
    stmr._vowels = set('aeiouy')  # noqa: SF01
    stmr_ee = Porter(early_english=True)
    stmr_ee._vowels = set('aeiouy')  # noqa: SF01

    def test_m_degree(self):
        """Test abydos.stemmer.Porter._m_degree."""
        # base case
        self.assertEqual(self.stmr._m_degree(''), 0)  # noqa: SF01

        # m==0
        self.assertEqual(self.stmr._m_degree('tr'), 0)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('ee'), 0)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('tree'), 0)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('y'), 0)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('by'), 0)  # noqa: SF01

        # m==1
        self.assertEqual(self.stmr._m_degree('trouble'), 1)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('oats'), 1)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('trees'), 1)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('ivy'), 1)  # noqa: SF01

        # m==2
        self.assertEqual(self.stmr._m_degree('troubles'), 2)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('private'), 2)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('oaten'), 2)  # noqa: SF01
        self.assertEqual(self.stmr._m_degree('orrery'), 2)  # noqa: SF01

    def test_has_vowel(self):
        """Test abydos.stemmer.Porter._has_vowel."""
        # base case
        self.assertFalse(self.stmr._has_vowel(''))  # noqa: SF01

        # False cases
        self.assertFalse(self.stmr._has_vowel('b'))  # noqa: SF01
        self.assertFalse(self.stmr._has_vowel('c'))  # noqa: SF01
        self.assertFalse(self.stmr._has_vowel('bc'))  # noqa: SF01
        self.assertFalse(
            self.stmr._has_vowel('bcdfghjklmnpqrstvwxYz')  # noqa: SF01
        )
        self.assertFalse(self.stmr._has_vowel('Y'))  # noqa: SF01

        # True cases
        self.assertTrue(self.stmr._has_vowel('a'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('e'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('ae'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('aeiouy'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('y'))  # noqa: SF01

        self.assertTrue(self.stmr._has_vowel('ade'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('cad'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('add'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('phi'))  # noqa: SF01
        self.assertTrue(self.stmr._has_vowel('pfy'))  # noqa: SF01

        self.assertFalse(self.stmr._has_vowel('pfY'))  # noqa: SF01

    def test_ends_in_doubled_cons(self):
        """Test abydos.stemmer.Porter._ends_in_doubled_cons."""
        # base case
        self.assertFalse(self.stmr._ends_in_doubled_cons(''))  # noqa: SF01

        # False cases
        self.assertFalse(self.stmr._ends_in_doubled_cons('b'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('c'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('bc'))  # noqa: SF01
        self.assertFalse(
            self.stmr._ends_in_doubled_cons(  # noqa: SF01
                'bcdfghjklmnpqrstvwxYz'
            )
        )
        self.assertFalse(self.stmr._ends_in_doubled_cons('Y'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('a'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('e'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('ae'))  # noqa: SF01
        self.assertFalse(
            self.stmr._ends_in_doubled_cons('aeiouy')  # noqa: SF01
        )
        self.assertFalse(self.stmr._ends_in_doubled_cons('y'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('ade'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('cad'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('phi'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('pfy'))  # noqa: SF01
        self.assertFalse(
            self.stmr._ends_in_doubled_cons('faddy')  # noqa: SF01
        )
        self.assertFalse(self.stmr._ends_in_doubled_cons('aiii'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_doubled_cons('ayyy'))  # noqa: SF01

        # True cases
        self.assertTrue(self.stmr._ends_in_doubled_cons('add'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_doubled_cons('fadd'))  # noqa: SF01
        self.assertTrue(
            self.stmr._ends_in_doubled_cons('fadddd')  # noqa: SF01
        )
        self.assertTrue(self.stmr._ends_in_doubled_cons('raYY'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_doubled_cons('doll'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_doubled_cons('parr'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_doubled_cons('parrr'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_doubled_cons('bacc'))  # noqa: SF01

    def test_ends_in_cvc(self):
        """Test abydos.stemmer.Porter._ends_in_cvc."""
        # base case
        self.assertFalse(self.stmr._ends_in_cvc(''))  # noqa: SF01

        # False cases
        self.assertFalse(self.stmr._ends_in_cvc('b'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('c'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('bc'))  # noqa: SF01
        self.assertFalse(
            self.stmr._ends_in_cvc('bcdfghjklmnpqrstvwxYz')  # noqa: SF01
        )
        self.assertFalse(self.stmr._ends_in_cvc('YYY'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('ddd'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('faaf'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('rare'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('rhy'))  # noqa: SF01

        # True cases
        self.assertTrue(self.stmr._ends_in_cvc('dad'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('phad'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('faded'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('maYor'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('enlil'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('parer'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('padres'))  # noqa: SF01
        self.assertTrue(self.stmr._ends_in_cvc('bacyc'))  # noqa: SF01

        # Special case for W, X, & Y
        self.assertFalse(self.stmr._ends_in_cvc('craw'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('max'))  # noqa: SF01
        self.assertFalse(self.stmr._ends_in_cvc('cray'))  # noqa: SF01

    def test_porter(self):
        """Test abydos.stemmer.Porter."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        # simple cases
        self.assertEqual(self.stmr.stem('c'), 'c')
        self.assertEqual(self.stmr.stem('da'), 'da')
        self.assertEqual(self.stmr.stem('ad'), 'ad')
        self.assertEqual(self.stmr.stem('sing'), 'sing')
        self.assertEqual(self.stmr.stem('singing'), 'sing')

        # missed branch test cases
        self.assertEqual(self.stmr.stem('capitalism'), 'capit')
        self.assertEqual(self.stmr.stem('fatalism'), 'fatal')
        self.assertEqual(self.stmr.stem('stional'), 'stional')
        self.assertEqual(self.stmr.stem('palism'), 'palism')
        self.assertEqual(self.stmr.stem('sization'), 'sizat')
        self.assertEqual(self.stmr.stem('licated'), 'licat')
        self.assertEqual(self.stmr.stem('lical'), 'lical')

    def test_porter_early_english(self):
        """Test abydos.stemmer.Porter (early English)."""
        # base case
        self.assertEqual(self.stmr_ee.stem(''), '')

        # simple cases (no different from regular stemmer)
        self.assertEqual(self.stmr_ee.stem('c'), 'c')
        self.assertEqual(self.stmr_ee.stem('da'), 'da')
        self.assertEqual(self.stmr_ee.stem('ad'), 'ad')
        self.assertEqual(self.stmr_ee.stem('sing'), 'sing')
        self.assertEqual(self.stmr_ee.stem('singing'), 'sing')

        # make
        self.assertEqual(self.stmr_ee.stem('make'), 'make')
        self.assertEqual(self.stmr_ee.stem('makes'), 'make')
        self.assertEqual(self.stmr_ee.stem('maketh'), 'make')
        self.assertEqual(self.stmr_ee.stem('makest'), 'make')

        # say
        self.assertEqual(self.stmr_ee.stem('say'), 'sai')
        self.assertEqual(self.stmr_ee.stem('says'), 'sai')
        self.assertEqual(self.stmr_ee.stem('sayeth'), 'sai')
        self.assertEqual(self.stmr_ee.stem('sayest'), 'sai')

        # missed branch test cases
        self.assertEqual(self.stmr_ee.stem('best'), 'best')
        self.assertEqual(self.stmr_ee.stem('meth'), 'meth')

    def test_porter_snowball(self):
        """Test abydos.stemmer.Porter (Snowball testset).

        These test cases are from
        http://snowball.tartarus.org/algorithms/porter/diffs.txt
        """
        #  Snowball Porter test set
        with open(_corpus_file('snowball_porter.csv')) as snowball_ts:
            next(snowball_ts)
            for line in snowball_ts:
                if line[0] != '#':
                    line = line.strip().split(',')
                    word, stem = line[0], line[1]
                    self.assertEqual(self.stmr.stem(word), stem.lower())


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_qgrams.

This module contains unit tests for abydos.tokenizer.QGrams
"""

import unittest

from abydos.tokenizer import WordpunctTokenizer


class WordpunctTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.WordpunctTokenizer."""

    def test_wordpunct_tokenizer(self):
        """Test abydos.tokenizer.WordpunctTokenizer."""
        self.assertEqual(
            sorted(WordpunctTokenizer().tokenize('').get_list()), []
        )
        self.assertEqual(
            sorted(WordpunctTokenizer().tokenize('a').get_list()), ['a']
        )

        self.assertEqual(
            sorted(WordpunctTokenizer().tokenize('NELSON').get_list()),
            sorted(['NELSON']),
        )
        self.assertEqual(
            sorted(WordpunctTokenizer().tokenize('NEILSEN').get_list()),
            sorted(['NEILSEN']),
        )

        tweet = 'I got a chance to catch up with the @Space_Station crew\
        today. Nothing like a call to space on #AstronomyNight!'
        self.assertEqual(
            sorted(WordpunctTokenizer().tokenize(tweet).get_list()),
            sorted(
                [
                    'I',
                    'got',
                    'a',
                    'chance',
                    'to',
                    'catch',
                    'up',
                    'with',
                    'the',
                    '@',
                    'Space_Station',
                    'crew',
                    'today',
                    '.',
                    'Nothing',
                    'like',
                    'a',
                    'call',
                    'to',
                    'space',
                    'on',
                    '#',
                    'AstronomyNight',
                    '!',
                ]
            ),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_koelner.

This module contains unit tests for abydos.phonetic.Koelner
"""

import unittest

from abydos.phonetic import Koelner


class KoelnerTestCases(unittest.TestCase):
    """Test Koelner Phonetic functions.

    test cases for abydos.phonetic.Koelner
    """

    pa = Koelner()

    def test_koelner_phonetik(self):
        """Test abydos.phonetic.Koelner."""
        self.assertEqual(self.pa.encode(''), '')

        # https://de.wikipedia.org/wiki/K%C3%B6lner_Phonetik
        self.assertEqual(self.pa.encode('Müller-Lüdenscheidt'), '65752682')
        self.assertEqual(self.pa.encode('Wikipedia'), '3412')
        self.assertEqual(self.pa.encode('Breschnew'), '17863')

        # http://search.cpan.org/~maros/Text-Phonetic/lib/Text/Phonetic/Koeln.pm
        self.assertEqual(self.pa.encode('Müller'), '657')
        self.assertEqual(self.pa.encode('schmidt'), '862')
        self.assertEqual(self.pa.encode('schneider'), '8627')
        self.assertEqual(self.pa.encode('fischer'), '387')
        self.assertEqual(self.pa.encode('weber'), '317')
        self.assertEqual(self.pa.encode('meyer'), '67')
        self.assertEqual(self.pa.encode('wagner'), '3467')
        self.assertEqual(self.pa.encode('schulz'), '858')
        self.assertEqual(self.pa.encode('becker'), '147')
        self.assertEqual(self.pa.encode('hoffmann'), '0366')
        self.assertEqual(self.pa.encode('schäfer'), '837')
        self.assertEqual(self.pa.encode('cater'), '427')
        self.assertEqual(self.pa.encode('axel'), '0485')

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('Akxel'), '0485')
        self.assertEqual(self.pa.encode('Adz'), '08')
        self.assertEqual(self.pa.encode('Alpharades'), '053728')
        self.assertEqual(self.pa.encode('Cent'), '862')
        self.assertEqual(self.pa.encode('Acre'), '087')
        self.assertEqual(self.pa.encode('H'), '')

    def test_koelner_phonetik_alpha(self):
        """Test abydos.phonetic.Koelner.encode_alpha."""
        self.assertEqual(
            self.pa.encode_alpha('Müller-Lüdenscheidt'), 'NLRLTNST'
        )
        self.assertEqual(self.pa.encode_alpha('Wikipedia'), 'FKPT')
        self.assertEqual(self.pa.encode_alpha('Breschnew'), 'PRSNF')
        self.assertEqual(self.pa.encode_alpha('Müller'), 'NLR')
        self.assertEqual(self.pa.encode_alpha('schmidt'), 'SNT')
        self.assertEqual(self.pa.encode_alpha('schneider'), 'SNTR')
        self.assertEqual(self.pa.encode_alpha('fischer'), 'FSR')
        self.assertEqual(self.pa.encode_alpha('weber'), 'FPR')
        self.assertEqual(self.pa.encode_alpha('meyer'), 'NR')
        self.assertEqual(self.pa.encode_alpha('wagner'), 'FKNR')
        self.assertEqual(self.pa.encode_alpha('schulz'), 'SLS')
        self.assertEqual(self.pa.encode_alpha('becker'), 'PKR')
        self.assertEqual(self.pa.encode_alpha('hoffmann'), 'AFNN')
        self.assertEqual(self.pa.encode_alpha('schäfer'), 'SFR')
        self.assertEqual(self.pa.encode_alpha('cater'), 'KTR')
        self.assertEqual(self.pa.encode_alpha('axel'), 'AKSL')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_sift4.

This module contains unit tests for abydos.distance.Sift4
"""

import unittest

from abydos.distance import Sift4


class Sift4TestCases(unittest.TestCase):
    """Test Sift4 functions.

    abydos.distance.Sift4
    """

    cmp = Sift4()
    cmp55 = Sift4(5, 5)

    def test_sift4_dist_abs(self):
        """Test abydos.distance.Sift4.dist_abs."""
        # tests copied from Lukas Benedix's post at
        # https://siderite.blogspot.com/2014/11/super-fast-and-accurate-string-distance.html
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 3)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 3)

        self.assertEqual(self.cmp.dist_abs('a', 'a'), 0)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0)

        self.assertEqual(self.cmp.dist_abs('a', 'ab'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'abc'), 1)
        self.assertEqual(self.cmp.dist_abs('abcdefg', 'xabxcdxxefxgx'), 7)

        self.assertEqual(self.cmp.dist_abs('ab', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'a'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('xabxcdxxefxgx', 'abcdefg'), 7)

        self.assertEqual(self.cmp.dist_abs('a', 'b'), 1)
        self.assertEqual(self.cmp.dist_abs('ab', 'ac'), 1)
        self.assertEqual(self.cmp.dist_abs('ac', 'bc'), 1)
        self.assertEqual(self.cmp.dist_abs('abc', 'axc'), 1)
        self.assertEqual(
            self.cmp.dist_abs('xabxcdxxefxgx', '1ab2cd34ef5g6'), 6
        )

        self.assertEqual(self.cmp.dist_abs('example', 'samples'), 2)
        self.assertEqual(self.cmp.dist_abs('sturgeon', 'urgently'), 3)
        self.assertEqual(self.cmp.dist_abs('levenshtein', 'frankenstein'), 6)
        self.assertEqual(self.cmp.dist_abs('distance', 'difference'), 5)

        # Tests copied from
        # https://github.com/tdebatty/java-string-similarity/blob/master/src/test/java/info/debatty/java/stringsimilarity/experimental/Sift4Test.java
        self.assertEqual(
            Sift4(5).dist_abs(
                'This is the first string', 'And this is another string'
            ),
            11,
        )
        self.assertEqual(
            Sift4(10).dist_abs(
                'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
                'Amet Lorm ispum dolor sit amet, consetetur adixxxpiscing'
                + ' elit.',
            ),
            12,
        )

        # cases with max_distance
        self.assertEqual(self.cmp55.dist_abs('example', 'samples'), 5)
        self.assertEqual(self.cmp55.dist_abs('sturgeon', 'urgently'), 5)
        self.assertEqual(self.cmp55.dist_abs('levenshtein', 'frankenstein'), 5)
        self.assertEqual(self.cmp55.dist_abs('distance', 'difference'), 5)

    def test_sift4_dist(self):
        """Test abydos.distance.Sift4.dist."""
        # tests copied from Lukas Benedix's post at
        # https://siderite.blogspot.com/2014/11/super-fast-and-accurate-string-distance.html
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('a', ''), 1)
        self.assertEqual(self.cmp.dist('', 'a'), 1)
        self.assertEqual(self.cmp.dist('abc', ''), 1)
        self.assertEqual(self.cmp.dist('', 'abc'), 1)

        self.assertEqual(self.cmp.dist('a', 'a'), 0)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0)

        self.assertEqual(self.cmp.dist('a', 'ab'), 0.5)
        self.assertEqual(self.cmp.dist('ac', 'abc'), 1 / 3)
        self.assertAlmostEqual(
            self.cmp.dist('abcdefg', 'xabxcdxxefxgx'), 0.538461538
        )

        self.assertEqual(self.cmp.dist('ab', 'b'), 0.5)
        self.assertEqual(self.cmp.dist('ab', 'a'), 0.5)
        self.assertEqual(self.cmp.dist('abc', 'ac'), 1 / 3)
        self.assertAlmostEqual(
            self.cmp.dist('xabxcdxxefxgx', 'abcdefg'), 0.538461538
        )

        self.assertEqual(self.cmp.dist('a', 'b'), 1)
        self.assertEqual(self.cmp.dist('ab', 'ac'), 0.5)
        self.assertEqual(self.cmp.dist('ac', 'bc'), 0.5)
        self.assertEqual(self.cmp.dist('abc', 'axc'), 1 / 3)
        self.assertAlmostEqual(
            self.cmp.dist('xabxcdxxefxgx', '1ab2cd34ef5g6'), 0.461538461
        )

        self.assertAlmostEqual(
            self.cmp.dist('example', 'samples'), 0.285714285
        )
        self.assertAlmostEqual(self.cmp.dist('sturgeon', 'urgently'), 0.375)
        self.assertAlmostEqual(
            self.cmp.dist('levenshtein', 'frankenstein'), 0.5
        )
        self.assertAlmostEqual(self.cmp.dist('distance', 'difference'), 0.5)

        # Tests copied from
        # https://github.com/tdebatty/java-string-similarity/blob/master/src/test/java/info/debatty/java/stringsimilarity/experimental/Sift4Test.java
        self.assertAlmostEqual(
            Sift4(5).dist(
                'This is the first string', 'And this is another string'
            ),
            0.423076923,
        )
        self.assertAlmostEqual(
            Sift4(10).dist(
                'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
                'Amet Lorm ispum dolor sit amet, consetetur adixxxpiscing'
                + ' elit.',
            ),
            0.193548387,
        )

        # cases with max_distance
        self.assertAlmostEqual(
            self.cmp55.dist('example', 'samples'), 0.714285714
        )
        self.assertAlmostEqual(self.cmp55.dist('sturgeon', 'urgently'), 0.625)
        self.assertAlmostEqual(
            self.cmp55.dist('levenshtein', 'frankenstein'), 0.416666666
        )
        self.assertAlmostEqual(self.cmp55.dist('distance', 'difference'), 0.5)

    def test_sift4_sim(self):
        """Test abydos.distance.Sift4.sim."""
        # tests copied from Lukas Benedix's post at
        # https://siderite.blogspot.com/2014/11/super-fast-and-accurate-string-distance.html
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('a', ''), 0)
        self.assertEqual(self.cmp.sim('', 'a'), 0)
        self.assertEqual(self.cmp.sim('abc', ''), 0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0)

        self.assertEqual(self.cmp.sim('a', 'a'), 1)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1)

        self.assertEqual(self.cmp.sim('a', 'ab'), 0.5)
        self.assertAlmostEqual(self.cmp.sim('ac', 'abc'), 2 / 3)
        self.assertAlmostEqual(
            self.cmp.sim('abcdefg', 'xabxcdxxefxgx'), 0.461538461
        )

        self.assertEqual(self.cmp.sim('ab', 'b'), 0.5)
        self.assertEqual(self.cmp.sim('ab', 'a'), 0.5)
        self.assertAlmostEqual(self.cmp.sim('abc', 'ac'), 2 / 3)
        self.assertAlmostEqual(
            self.cmp.sim('xabxcdxxefxgx', 'abcdefg'), 0.461538461
        )

        self.assertEqual(self.cmp.sim('a', 'b'), 0)
        self.assertEqual(self.cmp.sim('ab', 'ac'), 0.5)
        self.assertEqual(self.cmp.sim('ac', 'bc'), 0.5)
        self.assertAlmostEqual(self.cmp.sim('abc', 'axc'), 2 / 3)
        self.assertAlmostEqual(
            self.cmp.sim('xabxcdxxefxgx', '1ab2cd34ef5g6'), 0.538461538
        )

        self.assertAlmostEqual(self.cmp.sim('example', 'samples'), 0.714285714)
        self.assertAlmostEqual(self.cmp.sim('sturgeon', 'urgently'), 0.625)
        self.assertAlmostEqual(
            self.cmp.sim('levenshtein', 'frankenstein'), 0.5
        )
        self.assertAlmostEqual(self.cmp.sim('distance', 'difference'), 0.5)

        # Tests copied from
        # https://github.com/tdebatty/java-string-similarity/blob/master/src/test/java/info/debatty/java/stringsimilarity/experimental/Sift4Test.java
        self.assertAlmostEqual(
            Sift4(5).sim(
                'This is the first string', 'And this is another string'
            ),
            0.576923077,
        )
        self.assertAlmostEqual(
            Sift4(10).sim(
                'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
                'Amet Lorm ispum dolor sit amet, consetetur adixxxpiscing'
                + ' elit.',
            ),
            0.806451613,
        )

        # cases with max_distance
        self.assertAlmostEqual(
            self.cmp55.sim('example', 'samples'), 0.285714286
        )
        self.assertAlmostEqual(self.cmp55.sim('sturgeon', 'urgently'), 0.375)
        self.assertAlmostEqual(
            self.cmp55.sim('levenshtein', 'frankenstein'), 0.583333333
        )
        self.assertAlmostEqual(self.cmp55.sim('distance', 'difference'), 0.5)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.corpus.test_n_gram_corpus.

This module contains unit tests for abydos.corpus._n_gram_corpus
"""

import unittest
from collections import Counter

from abydos.corpus import Corpus
from abydos.corpus import NGramCorpus

from .. import _corpus_file


class NGramCorpusTestCases(unittest.TestCase):
    """Test abydos.corpus.NGramCorpus."""

    simple_corpus = NGramCorpus()
    simple_corpus.gng_importer(_corpus_file('simple-ngrams.txt'))

    double_corpus = NGramCorpus()
    double_corpus.gng_importer(_corpus_file('simple-ngrams.txt'))
    double_corpus.gng_importer(_corpus_file('simple-ngrams.txt'))

    sotu2015_sample = "Mr. Speaker, Mr. Vice President, Members of Congress,\
    my fellow Americans:\n\nWe are 15 years into this new century.\n Fifteen\
    years that dawned with terror touching our shores; that unfolded with a\
    new generation fighting two long and costly wars; that saw a vicious\
    recession spread across our nation and the world.\n It has been, and still\
    is, a hard time for many.\n\nBut tonight, we turn the page.\n Tonight,\
    after a breakthrough year for America, our economy is growing and creating\
    jobs at the fastest pace since 1999.\n Our unemployment rate is now lower\
    than it was before the financial crisis.\n More of our kids are graduating\
    than ever before.\n More of our people are insured than ever before.\n And\
    we are as free from the grip of foreign oil as we've been in almost 30\
    years.\n\nTonight, for the first time since 9/11, our combat mission in\
    Afghanistan is over.\n Six years ago, nearly 180,000 American troops\
    served in Iraq and Afghanistan.\n Today, fewer than 15,000 remain.\n And\
    we salute the courage and sacrifice of every man and woman in this 9/11\
    Generation who has served to keep us safe.\n We are humbled and grateful\
    for your service.\n\nAmerica, for all that we have endured; for all the\
    grit and hard work required to come back; for all the tasks that lie\
    ahead, know this: The shadow of crisis has passed, and the State of the\
    Union is strong.\n\nAt this moment -- with a growing economy, shrinking\
    deficits, bustling industry, booming energy production -- we have risen\
    from recession freer to write our own future than any other nation on\
    Earth.\n It's now up to us to choose who we want to be over the next 15\
    years and for decades to come.\n\nWill we accept an economy where only a\
    few of us do spectacularly well?\n Or will we commit ourselves to an\
    economy that generates rising incomes and chances for everyone who makes\
    the effort?\n\nWill we approach the world fearful and reactive, dragged\
    into costly conflicts that strain our military and set back our\
    standing?\n Or will we lead wisely, using all elements of our power to\
    defeat new threats and protect our planet?\n\nWill we allow ourselves to\
    be sorted into factions and turned against one another?\n Or will we\
    recapture the sense of common purpose that has always propelled America\
    forward?\n\nIn two weeks, I will send this Congress a budget filled with\
    ideas that are practical, not partisan.\n And in the months ahead, I'll\
    crisscross the country making a case for those ideas.\n So tonight, I want\
    to focus less on a checklist of proposals, and focus more on the values at\
    stake in the choices before us."
    sotu2015_corpus = Corpus(sotu2015_sample, filter_chars='.?-;,:')

    sotu_ngcorpus_uni = NGramCorpus(sotu2015_corpus)

    sotu_ngcorpus_tri = NGramCorpus()
    sotu_ngcorpus_tri.corpus_importer(sotu2015_corpus, 3, '<SOS>', '<EOS>')

    sotu_ngcorpus_5 = NGramCorpus()
    sotu_ngcorpus_5.corpus_importer(sotu2015_corpus, 5, '', '')

    simple_ngcorpus_5 = NGramCorpus()
    simple_ngcorpus_5.corpus_importer(
        Corpus(' '.join(['a'] * 10)), 15
    )  # 10 a's

    def test_init(self):
        """Test abydos.corpus.NGramCorpus.__init__."""
        self.assertIsInstance(NGramCorpus(), NGramCorpus)
        self.assertRaises(TypeError, NGramCorpus, ['a', 'b', 'c'])
        self.assertIsInstance(NGramCorpus(self.sotu2015_corpus), NGramCorpus)

    def test_corpus_importer(self):
        """Test abydos.corpus.NGramCorpus.corpus_importer."""
        self.assertRaises(
            TypeError, self.sotu_ngcorpus_5.corpus_importer, 'a b c d'
        )
        self.assertRaises(TypeError, self.sotu_ngcorpus_5.corpus_importer)

        self.assertIsInstance(self.sotu_ngcorpus_uni, NGramCorpus)
        self.assertIsInstance(self.sotu_ngcorpus_tri, NGramCorpus)

        self.assertIsInstance(self.sotu_ngcorpus_uni.ngcorpus, Counter)
        self.assertIsInstance(self.sotu_ngcorpus_tri.ngcorpus, Counter)

        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 1)), 10
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 2)), 9
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 3)), 8
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 4)), 7
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 5)), 6
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 6)), 5
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 7)), 4
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 8)), 3
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 9)), 2
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 10)), 1
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 11)), 0
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 12)), 0
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 13)), 0
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 14)), 0
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count(' '.join('a' * 15)), 0
        )
        self.assertEqual(self.simple_ngcorpus_5.get_count('_START_ a'), 1)
        self.assertEqual(self.simple_ngcorpus_5.get_count('a _END_'), 1)
        self.assertEqual(self.simple_ngcorpus_5.get_count('_END_ a'), 0)
        self.assertEqual(self.simple_ngcorpus_5.get_count('a _START_'), 0)
        self.assertEqual(
            self.simple_ngcorpus_5.get_count('_START_ a _END_'), 0
        )
        self.assertEqual(
            self.simple_ngcorpus_5.get_count('_END_ a _START_'), 0
        )

        self.assertEqual(self.sotu_ngcorpus_uni.get_count('Mr'), 2)
        self.assertEqual(self.sotu_ngcorpus_tri.get_count('Mr'), 2)

        self.assertEqual(self.sotu_ngcorpus_uni.get_count('the'), 19)
        self.assertEqual(self.sotu_ngcorpus_tri.get_count('the'), 19)

        self.assertEqual(self.sotu_ngcorpus_uni.get_count('to come'), 0)
        self.assertEqual(self.sotu_ngcorpus_tri.get_count('to come'), 2)

        self.assertEqual(self.sotu_ngcorpus_tri.get_count('<SOS> And'), 3)
        self.assertGreater(
            self.sotu_ngcorpus_tri.get_count('<SOS> And'),
            self.sotu_ngcorpus_5.get_count('<SOS> And'),
        )

    def test_gng_importer(self):
        """Test abydos.corpus.NGramCorpus.gng_importer."""
        self.assertIsInstance(self.simple_corpus, NGramCorpus)
        self.assertIsInstance(self.simple_corpus.ngcorpus, Counter)

        self.assertEqual(self.simple_corpus.get_count('the'), 20)
        self.assertEqual(self.double_corpus.get_count('the'), 40)

    def test_get_count(self):
        """Test abydos.corpus.NGramCorpus.get_count."""
        # string-style tests
        self.assertEqual(self.simple_corpus.get_count('the'), 20)
        self.assertEqual(self.simple_corpus.get_count('the quick'), 2)
        self.assertEqual(self.simple_corpus.get_count('trolley'), 0)

        # list-style tests
        self.assertEqual(self.simple_corpus.get_count(['the']), 20)
        self.assertEqual(self.simple_corpus.get_count(['the', 'quick']), 2)
        self.assertEqual(self.simple_corpus.get_count(['trolley']), 0)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.distance._pattern.

Pattern difference
"""

from typing import Any, Counter as TCounter, Optional, Sequence, Set, Union

from ._token_distance import _TokenDistance
from ..tokenizer import _Tokenizer

__all__ = ['Pattern']


class Pattern(_TokenDistance):
    r"""Pattern difference.

    For two sets X and Y and a population N, the pattern difference
    :cite:`Batagelj:1995`, Batagelj & Bren's :math:`- bc -` is

        .. math::

            dist_{pattern}(X, Y) =
            \frac{4 \cdot |X \setminus Y| \cdot |Y \setminus X|}
            {|N|^2}

    In :ref:`2x2 confusion table terms <confusion_table>`, where a+b+c+d=n,
    this is

        .. math::

            dist_{pattern} =
            \frac{4bc}{n^2}

    In :cite:`IBM:2017`, the formula omits the 4 in the numerator:
    :math:`\frac{bc}{n^2}`.

    .. versionadded:: 0.4.0
    """

    def __init__(
        self,
        alphabet: Optional[
            Union[TCounter[str], Sequence[str], Set[str], int]
        ] = None,
        tokenizer: Optional[_Tokenizer] = None,
        intersection_type: str = 'crisp',
        **kwargs: Any
    ) -> None:
        """Initialize Pattern instance.

        Parameters
        ----------
        alphabet : Counter, collection, int, or None
            This represents the alphabet of possible tokens.
            See :ref:`alphabet <alphabet>` description in
            :py:class:`_TokenDistance` for details.
        tokenizer : _Tokenizer
            A tokenizer instance from the :py:mod:`abydos.tokenizer` package
        intersection_type : str
            Specifies the intersection type, and set type as a result:
            See :ref:`intersection_type <intersection_type>` description in
            :py:class:`_TokenDistance` for details.
        **kwargs
            Arbitrary keyword arguments

        Other Parameters
        ----------------
        qval : int
            The length of each q-gram. Using this parameter and tokenizer=None
            will cause the instance to use the QGram tokenizer with this
            q value.
        metric : _Distance
            A string distance measure class for use in the ``soft`` and
            ``fuzzy`` variants.
        threshold : float
            A threshold value, similarities above which are counted as
            members of the intersection for the ``fuzzy`` variant.


        .. versionadded:: 0.4.0

        """
        super(Pattern, self).__init__(
            alphabet=alphabet,
            tokenizer=tokenizer,
            intersection_type=intersection_type,
            **kwargs
        )

    def dist(self, src: str, tar: str) -> float:
        """Return the Pattern difference of two strings.

        Parameters
        ----------
        src : str
            Source string (or QGrams/Counter objects) for comparison
        tar : str
            Target string (or QGrams/Counter objects) for comparison

        Returns
        -------
        float
            Pattern difference

        Examples
        --------
        >>> cmp = Pattern()
        >>> cmp.dist('cat', 'hat')
        2.6030820491461892e-05
        >>> cmp.dist('Niall', 'Neil')
        7.809246147438568e-05
        >>> cmp.dist('aluminum', 'Catalan')
        0.0003635035904093472
        >>> cmp.dist('ATCG', 'TAGC')
        0.0001626926280716368


        .. versionadded:: 0.4.0

        """
        self._tokenize(src, tar)

        b = self._src_only_card()
        c = self._tar_only_card()
        n = self._population_unique_card()

        num = b * c
        if num:
            return 4 * b * c / n ** 2
        return 0.0


if __name__ == '__main__':
    import doctest

    doctest.testmod()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_roger_root.

This module contains unit tests for abydos.phonetic.RogerRoot
"""

import unittest

from abydos.phonetic import RogerRoot


class RogerRootTestCases(unittest.TestCase):
    """Test Roger Root functions.

    test cases for abydos.phonetic.RogerRoot
    """

    pa = RogerRoot()

    def test_roger_root(self):
        """Test abydos.phonetic.RogerRoot."""
        self.assertEqual(self.pa.encode(''), '00000')

        # https://naldc.nal.usda.gov/download/27833/PDF
        self.assertEqual(self.pa.encode('BROWNER'), '09424')
        self.assertEqual(self.pa.encode('STANLEY'), '00125')
        self.assertEqual(self.pa.encode('CHALMAN'), '06532')
        self.assertEqual(self.pa.encode('CHING'), '06270')
        self.assertEqual(self.pa.encode('ANDERSON'), '12140')
        self.assertEqual(self.pa.encode('OVERSTREET'), '18401')
        self.assertEqual(self.pa.encode('HECKEL'), '27500')
        self.assertEqual(self.pa.encode('WYSZYNSKI'), '40207')
        self.assertEqual(self.pa.encode('WHITTED'), '41100')
        self.assertEqual(self.pa.encode('ONGOQO'), '12770')  # PDF had a typo?
        self.assertEqual(self.pa.encode('JOHNSON'), '32020')
        self.assertEqual(self.pa.encode('WILLIAMS'), '45300')
        self.assertEqual(self.pa.encode('SMITH'), '00310')
        self.assertEqual(self.pa.encode('JONES'), '32000')
        self.assertEqual(self.pa.encode('BROWN'), '09420')
        self.assertEqual(self.pa.encode('DAVIS'), '01800')
        self.assertEqual(self.pa.encode('JACKSON'), '37020')
        self.assertEqual(self.pa.encode('WILSON'), '45020')
        self.assertEqual(self.pa.encode('LEE'), '05000')
        self.assertEqual(self.pa.encode('THOMAS'), '01300')

        self.assertEqual(self.pa.encode('Defouw'), '01800')
        self.assertEqual(self.pa.encode('Dauphi'), '01800')
        self.assertEqual(self.pa.encode('Defazio'), '01800')
        self.assertEqual(self.pa.encode('Defay'), '01800')
        self.assertEqual(self.pa.encode('Davy'), '01800')
        self.assertEqual(self.pa.encode('Defee'), '01800')
        self.assertEqual(self.pa.encode('Dayhoff'), '01800')
        self.assertEqual(self.pa.encode('Davie'), '01800')
        self.assertEqual(self.pa.encode('Davey'), '01800')
        self.assertEqual(self.pa.encode('Davies'), '01800')
        self.assertEqual(self.pa.encode('Daves'), '01800')
        self.assertEqual(self.pa.encode('Deife'), '01800')
        self.assertEqual(self.pa.encode('Dehoff'), '01800')
        self.assertEqual(self.pa.encode('Devese'), '01800')
        self.assertEqual(self.pa.encode('Devoe'), '01800')
        self.assertEqual(self.pa.encode('Devee'), '01800')
        self.assertEqual(self.pa.encode('Devies'), '01800')
        self.assertEqual(self.pa.encode('Devos'), '01800')
        self.assertEqual(self.pa.encode('Dafoe'), '01800')
        self.assertEqual(self.pa.encode('Dove'), '01800')
        self.assertEqual(self.pa.encode('Duff'), '01800')
        self.assertEqual(self.pa.encode('Duffey'), '01800')
        self.assertEqual(self.pa.encode('Duffie'), '01800')
        self.assertEqual(self.pa.encode('Duffy'), '01800')
        self.assertEqual(self.pa.encode('Duyava'), '01800')
        self.assertEqual(self.pa.encode('Tafoya'), '01800')
        self.assertEqual(self.pa.encode('Tevis'), '01800')
        self.assertEqual(self.pa.encode('Tiffee'), '01800')
        self.assertEqual(self.pa.encode('Tivis'), '01800')
        self.assertEqual(self.pa.encode('Thevis'), '01800')
        self.assertEqual(self.pa.encode('Tovey'), '01800')
        self.assertEqual(self.pa.encode('Toeves'), '01800')
        self.assertEqual(self.pa.encode('Tuffs'), '01800')

        self.assertEqual(self.pa.encode('Samotid'), '00311')
        self.assertEqual(self.pa.encode('Simmet'), '00310')
        self.assertEqual(self.pa.encode('Simot'), '00310')
        self.assertEqual(self.pa.encode('Smead'), '00310')
        self.assertEqual(self.pa.encode('Smeda'), '00310')
        self.assertEqual(self.pa.encode('Smit'), '00310')
        self.assertEqual(self.pa.encode('Smite'), '00310')
        self.assertEqual(self.pa.encode('Smithe'), '00310')
        self.assertEqual(self.pa.encode('Smithey'), '00310')
        self.assertEqual(self.pa.encode('Smithson'), '00310')
        self.assertEqual(self.pa.encode('Smithy'), '00310')
        self.assertEqual(self.pa.encode('Smoot'), '00310')
        self.assertEqual(self.pa.encode('Smyth'), '00310')
        self.assertEqual(self.pa.encode('Szmodis'), '00310')
        self.assertEqual(self.pa.encode('Zemaitis'), '00310')
        self.assertEqual(self.pa.encode('Zmuda'), '00310')

        # Additional tests from @Yomguithereal's talisman
        # https://github.com/Yomguithereal/talisman/blob/master/test/phonetics/roger-root.js
        self.assertEqual(self.pa.encode('Guillaume'), '07530')
        self.assertEqual(self.pa.encode('Arlène'), '14520')
        self.assertEqual(self.pa.encode('Lüdenscheidt'), '05126')

        # no zero_pad
        nzp = RogerRoot(zero_pad=False)
        self.assertEqual(nzp.encode('BROWNER'), '09424')
        self.assertEqual(nzp.encode('STANLEY'), '00125')
        self.assertEqual(nzp.encode('CHALMAN'), '06532')
        self.assertEqual(nzp.encode('CHING'), '0627')
        self.assertEqual(nzp.encode('ANDERSON'), '12140')
        self.assertEqual(nzp.encode('OVERSTREET'), '18401')
        self.assertEqual(nzp.encode('HECKEL'), '275')
        self.assertEqual(nzp.encode('WYSZYNSKI'), '40207')
        self.assertEqual(nzp.encode('WHITTED'), '411')
        self.assertEqual(nzp.encode('ONGOQO'), '1277')
        self.assertEqual(nzp.encode('JOHNSON'), '3202')
        self.assertEqual(nzp.encode('WILLIAMS'), '4530')
        self.assertEqual(nzp.encode('SMITH'), '0031')
        self.assertEqual(nzp.encode('JONES'), '320')
        self.assertEqual(nzp.encode('BROWN'), '0942')
        self.assertEqual(nzp.encode('DAVIS'), '0180')
        self.assertEqual(nzp.encode('JACKSON'), '3702')
        self.assertEqual(nzp.encode('WILSON'), '4502')
        self.assertEqual(nzp.encode('LEE'), '05')
        self.assertEqual(nzp.encode('THOMAS'), '0130')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('BROWNER'), 'PRNR')
        self.assertEqual(self.pa.encode_alpha('STANLEY'), 'STNL')
        self.assertEqual(self.pa.encode_alpha('CHALMAN'), 'JLMN')
        self.assertEqual(self.pa.encode_alpha('CHING'), 'JNK')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_phonic.

This module contains unit tests for abydos.phonetic.PHONIC
"""

import unittest

from abydos.phonetic import PHONIC


class PHONICTestCases(unittest.TestCase):
    """Test PHONIC functions.

    test cases for abydos.phonetic.PHONIC
    """

    pa = PHONIC()

    def test_phonic(self):
        """Test abydos.phonetic.PHONIC."""
        self.assertEqual(self.pa.encode(''), '0000')

        # test case from paper
        self.assertEqual(self.pa.encode('Phillips'), 'P8590')

        # coverage
        self.assertEqual(
            PHONIC(max_length=-1, zero_pad=False, extended=True).encode(
                'Phillips'
            ),
            '8590',
        )
        self.assertEqual(
            PHONIC(max_length=-1, zero_pad=False, extended=True).encode_alpha(
                'Phillips'
            ),
            'FLPS',
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_q_grams.

This module contains unit tests for abydos.tokenizer.QGrams
"""

import sys
import unittest
from collections import Counter
from math import log1p

from abydos.tokenizer import QGrams


class QGramsTestCases(unittest.TestCase):
    """Test abydos.tokenizer.QGrams."""

    def test_qgrams(self):
        """Test abydos.tokenizer.QGrams."""
        self.assertEqual(sorted(QGrams().tokenize('').get_list()), [])
        self.assertEqual(
            sorted(QGrams(2).tokenize('a').get_list()), ['$a', 'a#']
        )
        self.assertEqual(sorted(QGrams(-1).tokenize('NELSON').get_list()), [])

        self.assertEqual(
            sorted(QGrams(3).tokenize('NELSON').get_list()),
            sorted(['$$N', '$NE', 'NEL', 'ELS', 'LSO', 'SON', 'ON#', 'N##']),
        )
        self.assertEqual(
            sorted(QGrams(7).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$$$$$$N',
                    '$$$$$NE',
                    '$$$$NEL',
                    '$$$NELS',
                    '$$NELSO',
                    '$NELSON',
                    'ELSON##',
                    'LSON###',
                    'N######',
                    'NELSON#',
                    'ON#####',
                    'SON####',
                ]
            ),
        )

        # http://www.sound-ex.com/alternative_qgram.htm
        self.assertEqual(
            sorted(QGrams().tokenize('NELSON').get_list()),
            sorted(['$N', 'NE', 'EL', 'LS', 'SO', 'ON', 'N#']),
        )
        self.assertEqual(
            sorted(QGrams().tokenize('NEILSEN').get_list()),
            sorted(['$N', 'NE', 'EI', 'IL', 'LS', 'SE', 'EN', 'N#']),
        )
        self.assertEqual(
            sorted(QGrams(start_stop='').tokenize('NELSON').get_list()),
            sorted(['NE', 'EL', 'LS', 'SO', 'ON']),
        )
        self.assertEqual(
            sorted(QGrams(start_stop='').tokenize('NEILSEN').get_list()),
            sorted(['NE', 'EI', 'IL', 'LS', 'SE', 'EN']),
        )

        # qval=(1,2)
        self.assertEqual(
            sorted(QGrams(qval=(1, 2)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$N',
                    'E',
                    'EL',
                    'L',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'O',
                    'ON',
                    'S',
                    'SO',
                ]
            ),
        )
        self.assertEqual(
            sorted(QGrams(qval=(2, 1)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$N',
                    'E',
                    'EL',
                    'L',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'O',
                    'ON',
                    'S',
                    'SO',
                ]
            ),
        )
        self.assertEqual(
            sorted(QGrams(qval=range(3)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$N',
                    'E',
                    'EL',
                    'L',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'O',
                    'ON',
                    'S',
                    'SO',
                ]
            ),
        )
        self.assertEqual(QGrams(qval=(1, 2)).tokenize('NELSON').count(), 13)

        # skip=(1,2)
        self.assertEqual(
            sorted(QGrams(skip=(2, 1, 0)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$E',
                    '$L',
                    '$N',
                    'EL',
                    'EO',
                    'ES',
                    'LN',
                    'LO',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'NL',
                    'NS',
                    'O',
                    'O#',
                    'ON',
                    'S#',
                    'SN',
                    'SO',
                ]
            ),
        )
        self.assertEqual(
            sorted(QGrams(skip=(2, 1, 0)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$E',
                    '$L',
                    '$N',
                    'EL',
                    'EO',
                    'ES',
                    'LN',
                    'LO',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'NL',
                    'NS',
                    'O',
                    'O#',
                    'ON',
                    'S#',
                    'SN',
                    'SO',
                ]
            ),
        )
        self.assertEqual(
            sorted(QGrams(skip=range(3)).tokenize('NELSON').get_list()),
            sorted(
                [
                    '$E',
                    '$L',
                    '$N',
                    'EL',
                    'EO',
                    'ES',
                    'LN',
                    'LO',
                    'LS',
                    'N',
                    'N',
                    'N#',
                    'NE',
                    'NL',
                    'NS',
                    'O',
                    'O#',
                    'ON',
                    'S#',
                    'SN',
                    'SO',
                ]
            ),
        )
        self.assertEqual(QGrams(skip=(0, 1, 2)).tokenize('NELSON').count(), 21)
        self.assertEqual(
            QGrams(qval=1).tokenize('COLIN').get_counter(),
            Counter({'C': 1, 'O': 1, 'L': 1, 'I': 1, 'N': 1}),
        )
        self.assertEqual(
            QGrams(qval=10, start_stop='').tokenize('COLIN').get_counter(),
            Counter({}),
        )
        if sys.version_info >= (3, 6):
            self.assertEqual(
                repr(QGrams(qval=1).tokenize('COLIN')),
                "QGrams({'C': 1, 'O': 1, 'L': 1, 'I': 1, 'N': 1})",
            )
        self.assertEqual(
            QGrams(qval=1).tokenize('COLIN').get_set(),
            {'C', 'O', 'L', 'I', 'N'},
        )

        # Test exception
        self.assertRaises(ValueError, QGrams, 0)

    def test_qgrams_intersections(self):
        """Test abydos.tokenizer.QGrams intersections."""
        self.assertEqual(
            sorted(QGrams().tokenize('NELSON') & QGrams().tokenize('')), []
        )
        self.assertEqual(
            sorted(QGrams().tokenize('') & QGrams().tokenize('NEILSEN')), []
        )
        self.assertEqual(
            sorted(QGrams().tokenize('NELSON') & QGrams().tokenize('NEILSEN')),
            sorted(['$N', 'NE', 'LS', 'N#']),
        )
        self.assertEqual(
            sorted(QGrams().tokenize('NELSON') & QGrams().tokenize('NOSLEN')),
            sorted(['$N', 'N#']),
        )
        self.assertEqual(
            sorted(QGrams().tokenize('NAIL') & QGrams().tokenize('LIAN')), []
        )

        self.assertEqual(
            sorted(
                QGrams(start_stop='').tokenize('NELSON')
                & QGrams(start_stop='').tokenize('NEILSEN')
            ),
            sorted(['NE', 'LS']),
        )
        self.assertEqual(
            sorted(
                QGrams(start_stop='').tokenize('NELSON')
                & QGrams(start_stop='').tokenize('NOSLEN')
            ),
            [],
        )
        self.assertEqual(
            sorted(
                QGrams(start_stop='').tokenize('NAIL')
                & QGrams(start_stop='').tokenize('LIAN')
            ),
            [],
        )

    def test_qgrams_counts(self):
        """Test abydos.tokenizer.QGrams counts."""
        self.assertEqual(QGrams().tokenize('').count(), 0)
        self.assertEqual(len(QGrams().tokenize('').get_list()), 0)

        self.assertEqual(QGrams().tokenize('NEILSEN').count(), 8)
        self.assertEqual(QGrams().tokenize('NELSON').count(), 7)
        self.assertEqual(QGrams(start_stop='').tokenize('NEILSEN').count(), 6)
        self.assertEqual(QGrams(start_stop='').tokenize('NELSON').count(), 5)

        self.assertEqual(len(QGrams().tokenize('NEILSEN').get_list()), 8)
        self.assertEqual(len(QGrams().tokenize('NELSON').get_list()), 7)
        self.assertEqual(
            len(QGrams(start_stop='').tokenize('NEILSEN').get_list()), 6
        )
        self.assertEqual(
            len(QGrams(start_stop='').tokenize('NELSON').get_list()), 5
        )

        self.assertEqual(
            QGrams(scaler='set').tokenize('ACAACACCTAG').get_counter(),
            Counter(
                {
                    '$A': 1,
                    'AC': 1,
                    'CA': 1,
                    'AA': 1,
                    'CC': 1,
                    'CT': 1,
                    'TA': 1,
                    'AG': 1,
                    'G#': 1,
                }
            ),
        )

        gold_standard = Counter(
            {
                '$A': 0.6931471805599453,
                'AC': 1.3862943611198906,
                'CA': 1.0986122886681096,
                'AA': 0.6931471805599453,
                'CC': 0.6931471805599453,
                'CT': 0.6931471805599453,
                'TA': 0.6931471805599453,
                'AG': 0.6931471805599453,
                'G#': 0.6931471805599453,
            }
        )
        test_counter = (
            QGrams(scaler=log1p).tokenize('ACAACACCTAG').get_counter()
        )
        for key in test_counter:
            self.assertAlmostEqual(test_counter[key], gold_standard[key])

        self.assertEqual(
            QGrams(scaler=log1p).tokenize('ACAACACCTAG').count_unique(), 9
        )

        tokens1 = QGrams().tokenize('ACAACACCTAG')
        tokens2 = QGrams().tokenize('GAAGATAC')
        self.assertEqual(
            tokens1 - tokens2,
            Counter({'$A': 1, 'AC': 2, 'CA': 2, 'CC': 1, 'CT': 1, 'G#': 1}),
        )
        self.assertEqual(
            tokens1 + tokens2,
            Counter(
                {
                    '$A': 1,
                    'AC': 4,
                    'CA': 2,
                    'AA': 2,
                    'CC': 1,
                    'CT': 1,
                    'TA': 2,
                    'AG': 2,
                    'G#': 1,
                    '$G': 1,
                    'GA': 2,
                    'AT': 1,
                    'C#': 1,
                }
            ),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_editex.

This module contains unit tests for abydos.distance.Editex
"""

import unittest

from abydos.distance import Editex


class EditexTestCases(unittest.TestCase):
    """Test Editex functions.

    abydos.distance.Editex
    """

    cmp = Editex()
    cmp_local = Editex(local=True)
    cmp_taper = Editex(taper=True)

    def test_editex_dist_abs(self):
        """Test abydos.distance.Editex.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('nelson', ''), 12)
        self.assertEqual(self.cmp.dist_abs('', 'neilsen'), 14)
        self.assertEqual(self.cmp.dist_abs('ab', 'a'), 2)
        self.assertEqual(self.cmp.dist_abs('ab', 'c'), 4)
        self.assertEqual(self.cmp.dist_abs('nelson', 'neilsen'), 2)
        self.assertEqual(self.cmp.dist_abs('neilsen', 'nelson'), 2)
        self.assertEqual(self.cmp.dist_abs('niall', 'neal'), 1)
        self.assertEqual(self.cmp.dist_abs('neal', 'niall'), 1)
        self.assertEqual(self.cmp.dist_abs('niall', 'nihal'), 2)
        self.assertEqual(self.cmp.dist_abs('nihal', 'niall'), 2)
        self.assertEqual(self.cmp.dist_abs('neal', 'nihl'), 3)
        self.assertEqual(self.cmp.dist_abs('nihl', 'neal'), 3)

        # Test tapering variant
        self.assertAlmostEqual(
            self.cmp_taper.dist_abs('nelson', 'neilsen'), 2.7142857143
        )

    def test_editex_dist_abs_local(self):
        """Test abydos.distance.Editex.dist_abs (local variant)."""
        self.assertEqual(self.cmp_local.dist_abs('', ''), 0)
        self.assertEqual(self.cmp_local.dist_abs('nelson', ''), 12)
        self.assertEqual(self.cmp_local.dist_abs('', 'neilsen'), 14)
        self.assertEqual(self.cmp_local.dist_abs('ab', 'a'), 2)
        self.assertEqual(self.cmp_local.dist_abs('ab', 'c'), 2)
        self.assertEqual(self.cmp_local.dist_abs('nelson', 'neilsen'), 2)
        self.assertEqual(self.cmp_local.dist_abs('neilsen', 'nelson'), 2)
        self.assertEqual(self.cmp_local.dist_abs('niall', 'neal'), 1)
        self.assertEqual(self.cmp_local.dist_abs('neal', 'niall'), 1)
        self.assertEqual(self.cmp_local.dist_abs('niall', 'nihal'), 2)
        self.assertEqual(self.cmp_local.dist_abs('nihal', 'niall'), 2)
        self.assertEqual(self.cmp_local.dist_abs('neal', 'nihl'), 3)
        self.assertEqual(self.cmp_local.dist_abs('nihl', 'neal'), 3)

    def test_editex_sim(self):
        """Test abydos.distance.Editex.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('nelson', ''), 0)
        self.assertEqual(self.cmp.sim('', 'neilsen'), 0)
        self.assertEqual(self.cmp.sim('ab', 'a'), 0.5)
        self.assertEqual(self.cmp.sim('ab', 'c'), 0)
        self.assertAlmostEqual(self.cmp.sim('nelson', 'neilsen'), 12 / 14)
        self.assertAlmostEqual(self.cmp.sim('neilsen', 'nelson'), 12 / 14)
        self.assertEqual(self.cmp.sim('niall', 'neal'), 0.9)

    def test_editex_dist(self):
        """Test abydos.distance.Editex.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('nelson', ''), 1)
        self.assertEqual(self.cmp.dist('', 'neilsen'), 1)
        self.assertEqual(self.cmp.dist('ab', 'a'), 0.5)
        self.assertEqual(self.cmp.dist('ab', 'c'), 1)
        self.assertAlmostEqual(self.cmp.dist('nelson', 'neilsen'), 2 / 14)
        self.assertAlmostEqual(self.cmp.dist('neilsen', 'nelson'), 2 / 14)
        self.assertEqual(self.cmp.dist('niall', 'neal'), 0.1)

        # Test tapering variant
        self.assertAlmostEqual(
            self.cmp_taper.dist('nelson', 'neilsen'), 0.123376623
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_lc_cutter.

This module contains unit tests for abydos.fingerprint.LCCutter
"""

import unittest

from abydos.fingerprint import LCCutter


class LCCutterTestCases(unittest.TestCase):
    """Test LCCutter functions.

    abydos.fingerprint.LCCutter
    """

    fp = LCCutter()

    def test_lc_cutter_fingerprint(self):
        """Test abydos.fingerprint.LCCutter."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '')
        self.assertEqual(self.fp.fingerprint('S'), 'S')

        # Test cases drawn from http://calculate.alptown.com/
        self.assertEqual(self.fp.fingerprint('Cutter'), 'C88847')
        self.assertEqual(self.fp.fingerprint('Quiet'), 'Q548')
        self.assertEqual(self.fp.fingerprint('Schmidt'), 'S36538')
        self.assertEqual(self.fp.fingerprint('Anderson'), 'A5347766')
        self.assertEqual(self.fp.fingerprint('Aziz'), 'A959')
        self.assertEqual(self.fp.fingerprint('I.B.M.'), 'I26')
        self.assertEqual(self.fp.fingerprint('Import'), 'I47678')
        self.assertEqual(self.fp.fingerprint('Sadron'), 'S23766')
        self.assertEqual(self.fp.fingerprint('Stinson'), 'S756766')
        self.assertEqual(self.fp.fingerprint('Cymbal'), 'C96335')
        self.assertEqual(self.fp.fingerprint('Ipswich'), 'I679534')
        self.assertEqual(self.fp.fingerprint('Rhododendron'), 'R46363463766')
        self.assertEqual(self.fp.fingerprint('Colin'), 'C6556')
        self.assertEqual(self.fp.fingerprint('Szelazek'), 'S9453945')
        self.assertEqual(self.fp.fingerprint('Quyen'), 'Q946')

        # Coverage
        self.assertEqual(self.fp.fingerprint('Qdoba'), 'Q2633')
        self.assertEqual(LCCutter(max_length=-1).fingerprint('Qdoba'), 'Q2633')
        self.assertEqual(LCCutter(max_length=3).fingerprint('Qdoba'), 'Q26')


if __name__ == '__main__':
    unittest.main()
